import tator

def test_clone_localization_type_same_host(host, token, project, image_type, video_type,
                                           box_type, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator.util.clone_media_type(tator_api, video_type, clone_project)
    dest_video_type = response.id
    response = tator.util.clone_media_type(tator_api, image_type, clone_project)
    dest_image_type = response.id
    response = tator.util.clone_localization_type(tator_api, box_type, clone_project,
                                                  {video_type: dest_video_type,
                                                   image_type: dest_image_type})
    assert(isinstance(response, tator.models.CreateResponse))

def test_clone_localization_type_different_host(host, token, project, image_type, video_type,
                                                box_type, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator.util.clone_media_type(tator_api, video_type, clone_project)
    dest_video_type = response.id
    response = tator.util.clone_media_type(tator_api, image_type, clone_project)
    dest_image_type = response.id
    response = tator.util.clone_localization_type(tator_api, box_type, clone_project,
                                                  {video_type: dest_video_type,
                                                   image_type: dest_image_type}, tator_api)
    assert(isinstance(response, tator.models.CreateResponse))




import tator

def test_version_crud(host, token, project):
    tator_api = tator.get_api(host, token)

    # Test single create.
    response = tator_api.create_version(project, version_spec={
        'name': 'Test Version',
        'description': 'A version for testing',
    })
    assert isinstance(response, tator.models.CreateResponse)
    print(response.message)
    pk = response.id

    # Test patch.
    response = tator_api.update_version(pk, version_update={'name': 'Updated Version'})
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)

    # Compare with get results.
    updated = tator_api.get_version(pk)
    assert isinstance(updated, tator.models.Version)
    assert updated.name == 'Updated Version'

    # Test delete.
    response = tator_api.delete_version(pk)
    assert isinstance(response, tator.models.MessageResponse)
    print(response.message)



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class AudioDefinition(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'bit_rate': 'int',
        'codec': 'str',
        'codec_description': 'str',
        'codec_mime': 'str',
        'host': 'str',
        'http_auth': 'str',
        'path': 'str',
        'size': 'int',
        'url': 'str'
    }

    attribute_map = {
        'bit_rate': 'bit_rate',
        'codec': 'codec',
        'codec_description': 'codec_description',
        'codec_mime': 'codec_mime',
        'host': 'host',
        'http_auth': 'http_auth',
        'path': 'path',
        'size': 'size',
        'url': 'url'
    }

    def __init__(self, bit_rate=None, codec=None, codec_description=None, codec_mime=None, host=None, http_auth=None, path=None, size=None, url=None, local_vars_configuration=None):  # noqa: E501
        """AudioDefinition - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._bit_rate = None
        self._codec = None
        self._codec_description = None
        self._codec_mime = None
        self._host = None
        self._http_auth = None
        self._path = None
        self._size = None
        self._url = None
        self.discriminator = None

        if bit_rate is not None:
            self.bit_rate = bit_rate
        self.codec = codec
        if codec_description is not None:
            self.codec_description = codec_description
        if codec_mime is not None:
            self.codec_mime = codec_mime
        if host is not None:
            self.host = host
        if http_auth is not None:
            self.http_auth = http_auth
        if path is not None:
            self.path = path
        if size is not None:
            self.size = size
        if url is not None:
            self.url = url

    @property
    def bit_rate(self):
        """
        Bit rate in bits per second

        :return: The bit_rate of this AudioDefinition. 
        :rtype: int
        """
        return self._bit_rate

    @bit_rate.setter
    def bit_rate(self, bit_rate):
        """
        Bit rate in bits per second

        :param bit_rate: The bit_rate of this AudioDefinition.
        :type: int
        """

        self._bit_rate = bit_rate

    @property
    def codec(self):
        """
        Human readable codec.

        :return: The codec of this AudioDefinition. 
        :rtype: str
        """
        return self._codec

    @codec.setter
    def codec(self, codec):
        """
        Human readable codec.

        :param codec: The codec of this AudioDefinition.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and codec is None:  # noqa: E501
            raise ValueError("Invalid value for `codec`, must not be `None`")  # noqa: E501

        self._codec = codec

    @property
    def codec_description(self):
        """
        Description other than codec.

        :return: The codec_description of this AudioDefinition. 
        :rtype: str
        """
        return self._codec_description

    @codec_description.setter
    def codec_description(self, codec_description):
        """
        Description other than codec.

        :param codec_description: The codec_description of this AudioDefinition.
        :type: str
        """

        self._codec_description = codec_description

    @property
    def codec_mime(self):
        """
        Example mime: \"video/mp4; codecs=\"avc1.64001e\"\". Only relevant for streaming files, will assume example above if not present.

        :return: The codec_mime of this AudioDefinition. 
        :rtype: str
        """
        return self._codec_mime

    @codec_mime.setter
    def codec_mime(self, codec_mime):
        """
        Example mime: \"video/mp4; codecs=\"avc1.64001e\"\". Only relevant for streaming files, will assume example above if not present.

        :param codec_mime: The codec_mime of this AudioDefinition.
        :type: str
        """

        self._codec_mime = codec_mime

    @property
    def host(self):
        """
        If supplied will use this instead of currently connected host, e.g. https://example.com

        :return: The host of this AudioDefinition. 
        :rtype: str
        """
        return self._host

    @host.setter
    def host(self, host):
        """
        If supplied will use this instead of currently connected host, e.g. https://example.com

        :param host: The host of this AudioDefinition.
        :type: str
        """

        self._host = host

    @property
    def http_auth(self):
        """
        If specified will be used for HTTP authorization in request for media, i.e. \"bearer <token>\".

        :return: The http_auth of this AudioDefinition. 
        :rtype: str
        """
        return self._http_auth

    @http_auth.setter
    def http_auth(self, http_auth):
        """
        If specified will be used for HTTP authorization in request for media, i.e. \"bearer <token>\".

        :param http_auth: The http_auth of this AudioDefinition.
        :type: str
        """

        self._http_auth = http_auth

    @property
    def path(self):
        """
        Relative URL to the file.

        :return: The path of this AudioDefinition. 
        :rtype: str
        """
        return self._path

    @path.setter
    def path(self, path):
        """
        Relative URL to the file.

        :param path: The path of this AudioDefinition.
        :type: str
        """

        self._path = path

    @property
    def size(self):
        """
        File size in bytes.

        :return: The size of this AudioDefinition. 
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """
        File size in bytes.

        :param size: The size of this AudioDefinition.
        :type: int
        """

        self._size = size

    @property
    def url(self):
        """
        Required when this object is used with the MoveVideo endpoint, this field is the upload URL of the audio file. It is is used for the src path and to create `path` field.

        :return: The url of this AudioDefinition. 
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """
        Required when this object is used with the MoveVideo endpoint, this field is the upload URL of the audio file. It is is used for the src path and to create `path` field.

        :param url: The url of this AudioDefinition.
        :type: str
        """

        self._url = url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AudioDefinition):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AudioDefinition):
            return True

        return self.to_dict() != other.to_dict()



def _convert_for_post(loc, localization_type_mapping, version_mapping, media_mapping):
    # Check for version mapping.
    version_id = loc.version
    if version_id in version_mapping:
        version_id = version_mapping[version_id]
    else:
        raise ValueError(f"Source version ID {version_id} missing from version_mapping!")
    # Check for media mapping.
    media_id = loc.media
    if media_id in media_mapping:
        media_id = media_mapping[media_id]
    else:
        raise ValueError(f"Source media ID {media_id} missing from media_mapping!")
    # Swap localization type IDs.
    localization_type_id = loc.meta
    if localization_type_id in localization_type_mapping:
        localization_type_id = localization_type_mapping[localization_type_id]
    else:
        raise ValueError(f"Source localization_type ID {localization_type_id} missing from "
                          "localization_type_mapping!")
    # Fill in required fields for post.
    return {'type': localization_type_id,
            'version': version_id,
            'media_id': media_id,
            'x': loc.x,
            'y': loc.y,
            'width': loc.width,
            'height': loc.height,
            'u': loc.u,
            'v': loc.v,
            'frame': loc.frame,
            **loc.attributes}

def clone_localization_list(src_api, query_params, dest_project, version_mapping, media_mapping,
                            localization_type_mapping, dest_api=None):
    """ Clone localization list.

    This can be used to clone localizations from one project to another or from one
    host to another.

    Example for different host:

    .. code-block:: python

        src_api = tator.get_api(host, token)
        dest_api = tator.get_api(other_host, other_token)
        query_params = {'project': 1, 'media_id': [1]}
        version_mapping = {1: 10} # Source version ID -> Dest version ID
        media_mapping = {1: 10} # Source media ID -> Dest media ID
        localization_type_mapping = {1: 10} # Source localization type ID -> Dest type ID
        dest_project = 1
        created_ids = []
        generator = clone_localization_list(src_api, query_params, dest_project, version_mapping,
                                            media_mapping, localization_type_mapping, dest_api)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} localizations...")
            created_ids.append(response.id)
        print(f"Finished creating {num_created} localizations!")

    Example for same host:

    .. code-block:: python

        api = tator.get_api(host, token)
        query_params = {'media_id': [1]}
        dest_project = 1
        version_mapping = {1: 10}
        media_mapping = {1: 10}
        localization_type_mapping = {1: 10} # Source localization type ID -> Dest type ID
        created_ids = []
        generator = clone_localization_list(src_api, query_params, dest_project, version_mapping,
                                            media_mapping, localization_type_mapping)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} localizations...")
            created_ids += response.id
        print(f"Finished creating {num_created} localizations!")

    :param src_api: :class:`tator.TatorApi` object corresponding to source host or only
        host if this is a clone on same host.
    :param query_params: Dictionary containing query parameters for source localization list.
    :param dest_project: Unique integer identifying destination project.
    :param version_mapping: Dictionary mapping source version IDs to destination version IDs. If the
        source localization list contains a version ID for which a destination version is not supplied, 
        an exception is raised.
    :param media_mapping: Dictionary mapping source media IDs to destination media IDs. If the
        source localization list contains a media ID for which a destination media is not supplied, 
        an exception is raised.
    :param localization_type_mapping: Dictionary mapping source localization type IDs to destination 
        localization_type IDs. If the source localization list contains a localization type ID for
        which a destination localization type ID is not supplied, an exception is raised.
    :param dest_api: :class:`tator.TatorApi` object corresponding to destination host.
    :returns: Generator containing number of localizations created, number of localizations total,
        most recent response from localization creation operation, and mapping between original IDs
        and created IDs.
    """
    # Make sure query has a project.
    if 'project' not in query_params:
        raise Exception("Query parameters must include a project!")

    # Set the dest_api if not given.
    if dest_api is None:
        dest_api = src_api

    # Start by getting list of localizations to be cloned.
    locs = src_api.get_localization_list(**query_params)

    # Convert to new spec.
    spec = [_convert_for_post(loc, localization_type_mapping, version_mapping, media_mapping)
            for loc in locs]

    # Create the localizations.
    created_ids = []
    total_localizations = len(spec)
    for idx in range(0, total_localizations, 500):
        response = dest_api.create_localization_list(dest_project,
                                                     localization_spec=spec[idx:idx+500])
        created_ids += response.id
        id_map = {src.id: dest_id for src, dest_id in zip(locs[idx:idx+500], response.id)}
        yield (len(created_ids), total_localizations, response, id_map)




""" Utility functions for interacting with the Tator platform """
from tator.util.chunked_create import chunked_create as chunked_create
from tator.util.get_api import get_api
from tator.util.chunked_create import chunked_create
from tator.util.download_media import download_media
from tator.util.download_temporary_file import download_temporary_file
from tator.util.upload_media import upload_media
from tator.util.upload_media_archive import upload_media_archive
from tator.util.chunked_file_list import chunked_file_list
from tator.util.upload_temporary_file import upload_temporary_file
from tator.util.md5sum import md5sum
from tator.util.multi_stream import make_multi_stream
from tator.util.get_images import get_images
from tator.util.full_state_graphic import full_state_graphic
from tator.util.to_dataframe import to_dataframe
from tator.util.register_algorithm import register_algorithm
from tator.util.clone_media_list import clone_media_list
from tator.util.clone_localization_list import clone_localization_list
from tator.util.clone_state_list import clone_state_list
from tator.util.clone_leaf_list import clone_leaf_list
from tator.util.clone_media_type import clone_media_type
from tator.util.clone_localization_type import clone_localization_type
from tator.util.clone_state_type import clone_state_type
from tator.util.clone_leaf_type import clone_leaf_type
from tator.util.clone_section import clone_section
from tator.util.clone_version import clone_version
from tator.util.clone_membership import clone_membership



import logging
import math
from uuid import uuid1

from tusclient.client import TusClient
from urllib.parse import urljoin
from progressbar import progressbar

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

def upload_file(path, api):
    host = api.api_client.configuration.host
    token = api.api_client.configuration.api_key['Authorization']
    prefix = api.api_client.configuration.api_key_prefix['Authorization']
    upload_uid = str(uuid1())
    tusURL = urljoin(host, "files/")
    tus = TusClient(tusURL, headers={'Authorization': f'{prefix} {token}',
                                     'Upload-Uid': f'{upload_uid}'})
    logger.info(f"Uploading file {path}...")
    chunk_size = 10*1024*1024 # 10 Mb
    uploader = tus.uploader(path, chunk_size=chunk_size,
                            retries=10, retry_delay=15)
    num_chunks = math.ceil(uploader.get_file_size()/chunk_size)

    for _ in progressbar(range(num_chunks)):
        uploader.upload_chunk()
    return uploader.url




# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class MediaSpec(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'object',
        'codec': 'str',
        'fps': 'int',
        'gid': 'str',
        'height': 'int',
        'md5': 'str',
        'name': 'str',
        'num_frames': 'int',
        'progress_name': 'str',
        'section': 'str',
        'thumbnail_gif_url': 'str',
        'thumbnail_url': 'str',
        'type': 'int',
        'uid': 'str',
        'url': 'str',
        'width': 'int'
    }

    attribute_map = {
        'attributes': 'attributes',
        'codec': 'codec',
        'fps': 'fps',
        'gid': 'gid',
        'height': 'height',
        'md5': 'md5',
        'name': 'name',
        'num_frames': 'num_frames',
        'progress_name': 'progress_name',
        'section': 'section',
        'thumbnail_gif_url': 'thumbnail_gif_url',
        'thumbnail_url': 'thumbnail_url',
        'type': 'type',
        'uid': 'uid',
        'url': 'url',
        'width': 'width'
    }

    def __init__(self, attributes=None, codec=None, fps=None, gid=None, height=None, md5=None, name=None, num_frames=None, progress_name=None, section=None, thumbnail_gif_url=None, thumbnail_url=None, type=None, uid=None, url=None, width=None, local_vars_configuration=None):  # noqa: E501
        """MediaSpec - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._codec = None
        self._fps = None
        self._gid = None
        self._height = None
        self._md5 = None
        self._name = None
        self._num_frames = None
        self._progress_name = None
        self._section = None
        self._thumbnail_gif_url = None
        self._thumbnail_url = None
        self._type = None
        self._uid = None
        self._url = None
        self._width = None
        self.discriminator = None

        self.attributes = attributes
        self.codec = codec
        self.fps = fps
        if gid is not None:
            self.gid = gid
        self.height = height
        self.md5 = md5
        self.name = name
        self.num_frames = num_frames
        if progress_name is not None:
            self.progress_name = progress_name
        self.section = section
        if thumbnail_gif_url is not None:
            self.thumbnail_gif_url = thumbnail_gif_url
        if thumbnail_url is not None:
            self.thumbnail_url = thumbnail_url
        self.type = type
        if uid is not None:
            self.uid = uid
        if url is not None:
            self.url = url
        self.width = width

    @property
    def attributes(self):
        """
        Attributes for the media

        :return: The attributes of this MediaSpec. 
        :rtype: object
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Attributes for the media

        :param attributes: The attributes of this MediaSpec.
        :type: object
        """

        self._attributes = attributes

    @property
    def codec(self):
        """
        Codec for videos.

        :return: The codec of this MediaSpec. 
        :rtype: str
        """
        return self._codec

    @codec.setter
    def codec(self, codec):
        """
        Codec for videos.

        :param codec: The codec of this MediaSpec.
        :type: str
        """

        self._codec = codec

    @property
    def fps(self):
        """
        Frame rate for videos.

        :return: The fps of this MediaSpec. 
        :rtype: int
        """
        return self._fps

    @fps.setter
    def fps(self, fps):
        """
        Frame rate for videos.

        :param fps: The fps of this MediaSpec.
        :type: int
        """

        self._fps = fps

    @property
    def gid(self):
        """
        Group ID for the upload group of this media.

        :return: The gid of this MediaSpec. 
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """
        Group ID for the upload group of this media.

        :param gid: The gid of this MediaSpec.
        :type: str
        """

        self._gid = gid

    @property
    def height(self):
        """
        Vertical resolution in pixels.

        :return: The height of this MediaSpec. 
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """
        Vertical resolution in pixels.

        :param height: The height of this MediaSpec.
        :type: int
        """

        self._height = height

    @property
    def md5(self):
        """
        MD5 sum of the media file.

        :return: The md5 of this MediaSpec. 
        :rtype: str
        """
        return self._md5

    @md5.setter
    def md5(self, md5):
        """
        MD5 sum of the media file.

        :param md5: The md5 of this MediaSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and md5 is None:  # noqa: E501
            raise ValueError("Invalid value for `md5`, must not be `None`")  # noqa: E501

        self._md5 = md5

    @property
    def name(self):
        """
        Name of the file.

        :return: The name of this MediaSpec. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the file.

        :param name: The name of this MediaSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def num_frames(self):
        """
        Number of frames for videos.

        :return: The num_frames of this MediaSpec. 
        :rtype: int
        """
        return self._num_frames

    @num_frames.setter
    def num_frames(self, num_frames):
        """
        Number of frames for videos.

        :param num_frames: The num_frames of this MediaSpec.
        :type: int
        """

        self._num_frames = num_frames

    @property
    def progress_name(self):
        """
        Replaces name in progress message.

        :return: The progress_name of this MediaSpec. 
        :rtype: str
        """
        return self._progress_name

    @progress_name.setter
    def progress_name(self, progress_name):
        """
        Replaces name in progress message.

        :param progress_name: The progress_name of this MediaSpec.
        :type: str
        """

        self._progress_name = progress_name

    @property
    def section(self):
        """
        Media section name.

        :return: The section of this MediaSpec. 
        :rtype: str
        """
        return self._section

    @section.setter
    def section(self, section):
        """
        Media section name.

        :param section: The section of this MediaSpec.
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and section is None:  # noqa: E501
            raise ValueError("Invalid value for `section`, must not be `None`")  # noqa: E501

        self._section = section

    @property
    def thumbnail_gif_url(self):
        """
        Upload URL for the video gif thumbnail if already generated.

        :return: The thumbnail_gif_url of this MediaSpec. 
        :rtype: str
        """
        return self._thumbnail_gif_url

    @thumbnail_gif_url.setter
    def thumbnail_gif_url(self, thumbnail_gif_url):
        """
        Upload URL for the video gif thumbnail if already generated.

        :param thumbnail_gif_url: The thumbnail_gif_url of this MediaSpec.
        :type: str
        """

        self._thumbnail_gif_url = thumbnail_gif_url

    @property
    def thumbnail_url(self):
        """
        Upload URL for the media thumbnail if already generated.

        :return: The thumbnail_url of this MediaSpec. 
        :rtype: str
        """
        return self._thumbnail_url

    @thumbnail_url.setter
    def thumbnail_url(self, thumbnail_url):
        """
        Upload URL for the media thumbnail if already generated.

        :param thumbnail_url: The thumbnail_url of this MediaSpec.
        :type: str
        """

        self._thumbnail_url = thumbnail_url

    @property
    def type(self):
        """
        Unique integer identifying a media type. Use -1 to automatically select the media type if only one media type exists in a project.

        :return: The type of this MediaSpec. 
        :rtype: int
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Unique integer identifying a media type. Use -1 to automatically select the media type if only one media type exists in a project.

        :param type: The type of this MediaSpec.
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and type is None:  # noqa: E501
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                type is not None and type < -1):  # noqa: E501
            raise ValueError("Invalid value for `type`, must be a value greater than or equal to `-1`")  # noqa: E501

        self._type = type

    @property
    def uid(self):
        """
        Unique ID for the upload of this media.

        :return: The uid of this MediaSpec. 
        :rtype: str
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """
        Unique ID for the upload of this media.

        :param uid: The uid of this MediaSpec.
        :type: str
        """

        self._uid = uid

    @property
    def url(self):
        """
        Upload URL for the image if this is an image type. If not an image, this field is ignored.

        :return: The url of this MediaSpec. 
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """
        Upload URL for the image if this is an image type. If not an image, this field is ignored.

        :param url: The url of this MediaSpec.
        :type: str
        """

        self._url = url

    @property
    def width(self):
        """
        Horizontal resolution in pixels.

        :return: The width of this MediaSpec. 
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """
        Horizontal resolution in pixels.

        :param width: The width of this MediaSpec.
        :type: int
        """

        self._width = width

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MediaSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MediaSpec):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class MediaTypeUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'archive_config': 'list[ArchiveConfig]',
        'description': 'str',
        'file_format': 'str',
        'keep_original': 'bool',
        'name': 'str',
        'streaming_config': 'list[ResolutionConfig]'
    }

    attribute_map = {
        'archive_config': 'archive_config',
        'description': 'description',
        'file_format': 'file_format',
        'keep_original': 'keep_original',
        'name': 'name',
        'streaming_config': 'streaming_config'
    }

    def __init__(self, archive_config=None, description='', file_format=None, keep_original=True, name=None, streaming_config=None, local_vars_configuration=None):  # noqa: E501
        """MediaTypeUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._archive_config = None
        self._description = None
        self._file_format = None
        self._keep_original = None
        self._name = None
        self._streaming_config = None
        self.discriminator = None

        if archive_config is not None:
            self.archive_config = archive_config
        if description is not None:
            self.description = description
        if file_format is not None:
            self.file_format = file_format
        if keep_original is not None:
            self.keep_original = keep_original
        if name is not None:
            self.name = name
        if streaming_config is not None:
            self.streaming_config = streaming_config

    @property
    def archive_config(self):
        """
        Archive config definitions. If null, the raw file will be uploaded to Tator.

        :return: The archive_config of this MediaTypeUpdate. 
        :rtype: list[ArchiveConfig]
        """
        return self._archive_config

    @archive_config.setter
    def archive_config(self, archive_config):
        """
        Archive config definitions. If null, the raw file will be uploaded to Tator.

        :param archive_config: The archive_config of this MediaTypeUpdate.
        :type: list[ArchiveConfig]
        """

        self._archive_config = archive_config

    @property
    def description(self):
        """
        Description of the media type.

        :return: The description of this MediaTypeUpdate. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the media type.

        :param description: The description of this MediaTypeUpdate.
        :type: str
        """

        self._description = description

    @property
    def file_format(self):
        """
        File extension. If omitted, any recognized file extension for the given dtype is accepted for upload. Do not include a dot prefix.

        :return: The file_format of this MediaTypeUpdate. 
        :rtype: str
        """
        return self._file_format

    @file_format.setter
    def file_format(self, file_format):
        """
        File extension. If omitted, any recognized file extension for the given dtype is accepted for upload. Do not include a dot prefix.

        :param file_format: The file_format of this MediaTypeUpdate.
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                file_format is not None and len(file_format) > 4):
            raise ValueError("Invalid value for `file_format`, length must be less than or equal to `4`")  # noqa: E501

        self._file_format = file_format

    @property
    def keep_original(self):
        """
        For video dtype, whether to keep the original video file for archival purposes after transcoding. If true, the originally uploaded file will be available for download, otherwise downloads will use the transcoded videos.

        :return: The keep_original of this MediaTypeUpdate. 
        :rtype: bool
        """
        return self._keep_original

    @keep_original.setter
    def keep_original(self, keep_original):
        """
        For video dtype, whether to keep the original video file for archival purposes after transcoding. If true, the originally uploaded file will be available for download, otherwise downloads will use the transcoded videos.

        :param keep_original: The keep_original of this MediaTypeUpdate.
        :type: bool
        """

        self._keep_original = keep_original

    @property
    def name(self):
        """
        Name of the media type.

        :return: The name of this MediaTypeUpdate. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the media type.

        :param name: The name of this MediaTypeUpdate.
        :type: str
        """

        self._name = name

    @property
    def streaming_config(self):
        """
        Streaming config defintion. If null, the default will be used.

        :return: The streaming_config of this MediaTypeUpdate. 
        :rtype: list[ResolutionConfig]
        """
        return self._streaming_config

    @streaming_config.setter
    def streaming_config(self, streaming_config):
        """
        Streaming config defintion. If null, the default will be used.

        :param streaming_config: The streaming_config of this MediaTypeUpdate.
        :type: list[ResolutionConfig]
        """

        self._streaming_config = streaming_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MediaTypeUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MediaTypeUpdate):
            return True

        return self.to_dict() != other.to_dict()



import math
import os
import requests
import subprocess
import tempfile

from tator.transcode.upload import upload_file
import tator

def _download_file(headers, url, out_path):
    CHUNK_SIZE=2*1024*1024
    with requests.get(url, stream=True, headers=headers) as r:
        r.raise_for_status()
        total_size = r.headers['Content-Length']
        total_chunks = math.ceil(int(total_size) / CHUNK_SIZE)
        chunk_count = 0
        last_progress = 0
        with open(out_path, 'wb') as f:
            for chunk in r.iter_content(chunk_size=CHUNK_SIZE):
                if chunk:
                    chunk_count += 1
                    f.write(chunk)

def make_multi_stream(api, type_id, layout,name, media_ids,section=None,quality=None):
    """ Uploads a single media file.

    :param api: :class:`tator.TatorApi` object.
    :param type_id: Unique integer identifying a multi-stream media type.
    :param layout: Path to the media file.
    :param name: Name of the file to use
    :param media_ids: List of media_ids to multi-stream
    :param quality: [Optional] Media section to upload to.
    :param section [Optional]: Section attribute to apply to media element (tator_user_sections value, typically a UUID).
    :returns: Boolean representing success (True means success)
    """

    # Fetch the stuff we need to download files
    config = api.api_client.configuration
    host = config.host
    token = config.api_key['Authorization']
    prefix = config.api_key_prefix['Authorization']

    # use a multi extension
    name += ".multi"

    # Fetch the media type
    multi_stream_type = api.get_media_type(type_id)
    project = multi_stream_type.project

    assert(len(media_ids) == layout[0]*layout[1])

    media_objects = api.get_media_list(project,media_id=media_ids)
    assert(len(media_objects) == len(media_ids))

    media_lookup={}
    for media in media_objects:
        media_lookup[media.id] = media

    attributes={}
    if section:
        attributes.update({"tator_user_sections": section})

    headers = {
        'Authorization': f'{prefix} {token}',
        'Content-Type': f'application/json',
        'Accept-Encoding': 'gzip',
    }

    # Download the thumbnails into a temporary
    with tempfile.TemporaryDirectory() as d:
        for pos,media_id in enumerate(media_ids):
            media = media_lookup[media_id]
            thumb = host + "/media/" + media.thumbnail
            thumb_gif = host + "/media/" + media.thumbnail_gif
            _download_file(headers, thumb, os.path.join(d,
                                                        f"thumb_{pos:09d}.jpg"))
            _download_file(headers, thumb_gif, os.path.join(d,
                                                        f"gif_{pos:09d}.gif"))

        cmd = ["ffmpeg",
               "-y",
               "-i", "thumb_%09d.jpg",
               "-vf",f"tile={layout[1]}x{layout[0]},scale=256:-1",
               os.path.join(d,"tiled_thumb.jpg")]
        subprocess.run(cmd,cwd=d,check=True)

        input_files=[]
        rows=layout[0]
        cols=layout[1]
        filter_graph=""
        idx = 0
        for row in range(rows):
            for col in range(cols):
                filter_graph += f"[{idx}:v]"
                idx+=1
                if cols > 1 and col + 1 == cols:
                    filter_graph += f"hstack=inputs={cols}[r{row}];"
        for row in range(rows):
            if cols > 1:
                filter_graph += f"[r{row}]"
        if rows > 1 and row + 1 == rows:
            filter_graph+=f'vstack=inputs={rows}[tiled_gif];[tiled_gif]'
        filter_graph+=f'scale=256:-1[raw];[raw]split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse[final]'
        print(filter_graph)
        for pos,_ in enumerate(media_ids):
            input_files.extend(['-i', f'gif_{pos:09d}.gif'])
        cmd = ["ffmpeg",
               "-y",
               *input_files,
               "-filter_complex", filter_graph,
               "-map", "[final]",
               "-shortest",
               "tiled_gif.gif"]

        subprocess.run(cmd,cwd=d,check=True)

        thumbnail_url = upload_file(os.path.join(d,'tiled_thumb.jpg'),
                                    api)
        thumbnail_gif_url = upload_file(os.path.join(d,'tiled_gif.gif'),
                                        api)

        md5=tator.util.md5sum(os.path.join(d,'tiled_gif.gif'))

        section_name = None
        if section:
            sections = api.get_section_list(project)
            for this_section in sections:
                if this_section.tator_user_sections == section:
                    section_name = this_section.name
                    break

        media_spec = {'attributes':attributes,
                      'name':name,
                      'thumbnail_url':thumbnail_url,
                      'md5': md5,
                      'section':section_name,
                      'type':type_id}

        resp = api.create_media(project,media_spec)

        print(f"Created {resp.id}")
        media_files={"layout": layout,
                     "ids": media_ids}
        if quality:
            media_files.update({"quality": quality})
        api.update_media(resp.id,
                         {"thumbnail_gif_url": thumbnail_gif_url,
                          "thumbnail_url": thumbnail_url,
                          "media_files": media_files})

















# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Algorithm(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster': 'int',
        'description': 'str',
        'files_per_job': 'int',
        'id': 'int',
        'manifest': 'str',
        'name': 'str',
        'project': 'int',
        'user': 'int'
    }

    attribute_map = {
        'cluster': 'cluster',
        'description': 'description',
        'files_per_job': 'files_per_job',
        'id': 'id',
        'manifest': 'manifest',
        'name': 'name',
        'project': 'project',
        'user': 'user'
    }

    def __init__(self, cluster=None, description=None, files_per_job=None, id=None, manifest=None, name=None, project=None, user=None, local_vars_configuration=None):  # noqa: E501
        """Algorithm - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._cluster = None
        self._description = None
        self._files_per_job = None
        self._id = None
        self._manifest = None
        self._name = None
        self._project = None
        self._user = None
        self.discriminator = None

        if cluster is not None:
            self.cluster = cluster
        if description is not None:
            self.description = description
        if files_per_job is not None:
            self.files_per_job = files_per_job
        if id is not None:
            self.id = id
        if manifest is not None:
            self.manifest = manifest
        if name is not None:
            self.name = name
        if project is not None:
            self.project = project
        if user is not None:
            self.user = user

    @property
    def cluster(self):
        """
        Unique integer identifying the job cluster.

        :return: The cluster of this Algorithm. 
        :rtype: int
        """
        return self._cluster

    @cluster.setter
    def cluster(self, cluster):
        """
        Unique integer identifying the job cluster.

        :param cluster: The cluster of this Algorithm.
        :type: int
        """

        self._cluster = cluster

    @property
    def description(self):
        """
        Description of the algorithm.

        :return: The description of this Algorithm. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the algorithm.

        :param description: The description of this Algorithm.
        :type: str
        """

        self._description = description

    @property
    def files_per_job(self):
        """
        Number of media files to be submitted to each workflow.

        :return: The files_per_job of this Algorithm. 
        :rtype: int
        """
        return self._files_per_job

    @files_per_job.setter
    def files_per_job(self, files_per_job):
        """
        Number of media files to be submitted to each workflow.

        :param files_per_job: The files_per_job of this Algorithm.
        :type: int
        """

        self._files_per_job = files_per_job

    @property
    def id(self):
        """
        Unique integer identifying the registered algorithm.

        :return: The id of this Algorithm. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying the registered algorithm.

        :param id: The id of this Algorithm.
        :type: int
        """

        self._id = id

    @property
    def manifest(self):
        """
        Server URL to argo manifest file (.yaml)

        :return: The manifest of this Algorithm. 
        :rtype: str
        """
        return self._manifest

    @manifest.setter
    def manifest(self, manifest):
        """
        Server URL to argo manifest file (.yaml)

        :param manifest: The manifest of this Algorithm.
        :type: str
        """

        self._manifest = manifest

    @property
    def name(self):
        """
        Unique name of the algorithm workflow.

        :return: The name of this Algorithm. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Unique name of the algorithm workflow.

        :param name: The name of this Algorithm.
        :type: str
        """

        self._name = name

    @property
    def project(self):
        """
        Unique integer identifying the project associated with the algorithm.

        :return: The project of this Algorithm. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying the project associated with the algorithm.

        :param project: The project of this Algorithm.
        :type: int
        """

        self._project = project

    @property
    def user(self):
        """
        Unique integer identifying the user registering the algorithm.

        :return: The user of this Algorithm. 
        :rtype: int
        """
        return self._user

    @user.setter
    def user(self, user):
        """
        Unique integer identifying the user registering the algorithm.

        :param user: The user of this Algorithm.
        :type: int
        """

        self._user = user

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Algorithm):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Algorithm):
            return True

        return self.to_dict() != other.to_dict()



import math
import logging

import requests

logger = logging.getLogger(__name__)

def _download_file(url, headers, out_path):
    CHUNK_SIZE = 2 * 1024 * 1024
    MAX_RETRIES = 10
    for attempt in range(MAX_RETRIES):
        try:
            with requests.get(url, stream=True, headers=headers) as r:
                r.raise_for_status()
                total_size = r.headers['Content-Length']
                total_chunks = math.ceil(int(total_size) / CHUNK_SIZE)
                chunk_count = 0
                last_progress = 0
                yield last_progress
                with open(out_path, 'wb') as f:
                    for chunk in r.iter_content(chunk_size=CHUNK_SIZE):
                        if chunk:
                            chunk_count += 1
                            f.write(chunk)
                            this_progress = round((chunk_count / total_chunks) *100,1)
                            if this_progress != last_progress:
                                yield this_progress
                                last_progress = this_progress
                yield 100
            break
        except Exception as ex:
            logger.info(f"Failed to download {url} on attempt {attempt}...")
            pass




import os
import glob
import tarfile
import tempfile
import random
import uuid
import datetime

import tator
from .test_localization import random_localization

def random_localization(video_obj):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
    }
    out = {
        'x': x,
        'y': y,
        'width': w,
        'height': h,
        'frame': random.randint(0, video_obj.num_frames - 1),
    }
    return {**out, **attributes}

def random_state(video_obj):
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
    }
    out = {
        'frame': random.randint(0, video_obj.num_frames - 1),
    }
    return {**out, **attributes}

def test_upload_file_list(host, token, project, image_type, image_set):
    tator_api = tator.get_api(host, token)
    paths = os.listdir(image_set)
    paths = [os.path.join(image_set, path) for path in paths]
    paths = paths[:100] # Only upload the first 100 files.
    batch_num = 0
    for batch in tator.util.chunked_file_list(paths):
        print(f"Uploading file {batch_num*10} / {len(paths)}")
        for progress, response in tator.util.upload_media_archive(tator_api, project, batch):
            print(f"Archive upload progress: {progress}%")
        batch_num += 1
        assert isinstance(response, tator.models.Transcode)
        print(response.message)
    
def test_upload_archive(host, token, project, image_type, image_set, video_type, video_file, video,
                        box_type, state_type):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(video)
    video_base = os.path.basename(video_file)

    # Add images to tar file.
    paths = glob.glob(os.path.join(image_set, '**/*.jpg'), recursive=True)
    paths = paths[:10] # Only upload the first 10 files.
    tar_buf = tempfile.NamedTemporaryFile()
    tar_file = tarfile.TarFile(mode='w', fileobj=tar_buf)
    for idx,fp in enumerate(paths):
        tar_file.add(fp, arcname=os.path.basename(fp))

    # Add video to tar file.
    tar_file.add(video_file, arcname=video_base)

    # Make csv with localizations in it.
    with tempfile.NamedTemporaryFile('w') as fp:
        fp.write('frame,x,y,width,height,test_string\n')
        for _ in range(10):
            loc = random_localization(video_obj)
            fp.write(f"{loc['frame']},{loc['x']},{loc['x']},{loc['width']},"
                     f"{loc['height']},{loc['test_string']}\n")
        arcname = os.path.join(os.path.splitext(video_base)[0], 'localizations',
                                                f"{box_type}.csv")
        fp.flush()
        tar_file.add(fp.name, arcname=arcname)

    # Make csv with states in it.
    with tempfile.NamedTemporaryFile('w') as fp:
        fp.write('frame,test_string\n')
        for _ in range(10):
            loc = random_state(video_obj)
            fp.write(f"{loc['frame']},{loc['test_string']}\n")
        arcname = os.path.join(os.path.splitext(video_base)[0], 'states',
                                                f"{state_type}.csv")
        fp.flush()
        tar_file.add(fp.name, arcname=arcname)

    tar_buf.flush()

    # Upload the archive.
    for progress, response in tator.util.upload_media_archive(tator_api, project, tar_buf.name):
        print(f"Archive upload progress: {progress}%")
    assert isinstance(response, tator.models.Transcode)
    print(response.message)
    



#!/usr/bin/env python

import argparse
import logging
import subprocess
import json
import os
import sys

from ..util.get_api import get_api
from ..openapi.tator_openapi.models import CreateResponse

from .upload import upload_file
from .make_fragment_info import make_fragment_info

logger = logging.getLogger(__name__)

# If HW is available, use this as lookup swap
encoder_lookup=None

def find_best_encoder(codec):
    """ Find the best encoder based on what is available on the system """
    global encoder_lookup
    if encoder_lookup is None:
        # Default codecs
        encoder_lookup={"hevc": "libsvt_hevc",
                        "h264": "libx264"}
        cmd = [
            "ffmpeg",
            "-encoders" ]
        output=subprocess.run(cmd,stdout=subprocess.PIPE,check=True).stdout.decode()
        if output.find("hevc_qsv") >= 0:
            encoder_lookup["hevc"] = "hevc_qsv"
        if output.find("h264_qsv") >= 0:
            encoder_lookup["h264"] = "h264_qsv"
        print(f"encoder_lookup = {encoder_lookup}")
    return encoder_lookup.get(codec,codec)

def parse_args():
    parser = argparse.ArgumentParser(description='Transcodes a raw video.')
    parser.add_argument('--host', type=str, default='https://www.tatorapp.com', help='Host URL.')
    parser.add_argument('--token', type=str, help='REST API token.')
    parser.add_argument('--media', type=int, help='Unique integer identifying a media.')
    parser.add_argument('--category', required=True, help='One of streaming, archival, or audio.')
    parser.add_argument('--raw_width', type=int, help='Pixel width of original video.')
    parser.add_argument('--raw_height', type=int, help='Pixel height of original video.')
    parser.add_argument('--configs', type=str, help='Comma separated list of output configs, '
                                                    'format is resolution:crf:codec.')
    parser.add_argument('--input', type=str, help='Path to raw video.')
    parser.add_argument("-o", "--output")
    return parser.parse_args()

def make_video_definition(disk_file):
    cmd = [
        "ffprobe",
        "-v","error",
        "-show_entries", "stream",
        "-print_format", "json",
        "-select_streams", "v",
        disk_file,
    ]
    output = subprocess.run(cmd, stdout=subprocess.PIPE, check=True).stdout
    video_info = json.loads(output)
    stream_idx=0
    for idx, stream in enumerate(video_info["streams"]):
        if stream["codec_type"] == "video":
            stream_idx=idx
            break
    stream = video_info["streams"][stream_idx]
    video_def = {"resolution": (stream["height"], stream["width"]),
                 "codec": stream["codec_name"],
                 "codec_description": stream["codec_long_name"],
                 "size": os.stat(disk_file).st_size,
                 "bit_rate": int(stream.get("bit_rate",-1))}
    return video_def

def convert_streaming(host, token, media, path, outpath, raw_width, raw_height, configs):
    print(f"Transcoding {path} to {outpath}...")
    # Get workload parameters.
    os.makedirs(outpath, exist_ok=True)

    # Convert settings into resolution/crf/codec.
    resolutions = [int(config.split(':')[0]) for config in configs]
    crfs = [config.split(':')[1] for config in configs]
    codecs = [config.split(':')[2] for config in configs]

    # Need to get avg_framerate
    cmd = [
        "ffprobe",
        "-v","error",
        "-show_entries", "stream",
        "-print_format", "json",
        "-select_streams", "v",
        path,
    ]
    output = subprocess.run(cmd, stdout=subprocess.PIPE, check=True).stdout
    video_info = json.loads(output)
    avg_frame_rate=video_info['streams'][0]['avg_frame_rate']

    vid_dims = [raw_height, raw_width]
    cmd = [
        "ffmpeg", "-y",
        "-i", path,
        "-i", os.path.join(os.path.dirname(os.path.abspath(__file__)), "black.mp4"),
    ]

    per_res = ["-an",
        "-metadata:s", "handler_name=tator",
        "-g", "25",
        "-preset", "fast",
        "-movflags",
        "faststart+frag_keyframe+empty_moov+default_base_moof",
        "-tune", "fastdecode",]

    print(f"Transcoding to {resolutions}")
    for ridx, resolution in enumerate(resolutions):
        logger.info(f"Generating resolution @ {resolution}")
        output_file = os.path.join(outpath, f"{resolution}.mp4")
        codec = find_best_encoder(codecs[ridx])
        quality_flag = "-crf"
        pixel_format = "yuv420p"
        if codec.find("qsv") >= 0:
            quality_flag = "-global_quality"
            pixel_format = "nv12"
        cmd.extend([*per_res,
                    "-vcodec", codec,
                    "-pix_fmt", pixel_format,
                    quality_flag, crfs[ridx],
                    "-filter_complex",
                    # Scale the black mp4 to the input resolution prior to concating and scaling back down.
                    f"[0:v:0]yadif[a{ridx}];[a{ridx}]setsar=1[vid{ridx}];[1:v:0]scale={vid_dims[1]}:{vid_dims[0]},setsar=1[bv{ridx}];[vid{ridx}][bv{ridx}]concat=n=2:v=1:a=0[rv{ridx}];[rv{ridx}]scale=-2:{resolution}[catv{ridx}];[catv{ridx}]pad=ceil(iw/2)*2:ceil(ih/2)*2[norate{ridx}];[norate{ridx}]fps={avg_frame_rate}[outv{ridx}]",
                    "-map", f"[outv{ridx}]",
                    output_file])

    logger.info('ffmpeg cmd = {}'.format(cmd))
    subprocess.run(cmd, check=True)
    api = get_api(host, token)

    for resolution in resolutions:
        output_file = os.path.join(outpath, f"{resolution}.mp4")

        segments_file = os.path.join(outpath, f"{resolution}.json")
        make_fragment_info(output_file, segments_file)

        logger.info("Uploading transcoded file...")
        url = upload_file(output_file, api)

        logger.info("Uploading segments file...")
        segments_url = upload_file(segments_file, api)

        # Construct move video spec.
        spec = {
            'media_files': {'streaming': [{
                **make_video_definition(output_file),
                'url': url,
                'segments_url': segments_url,
            }]}
        }

        # Move video file with the api.
        response = api.move_video(media, move_video_spec=spec)
        assert isinstance(response, CreateResponse)

def default_archival_upload(api, host, media, path, encoded):
    # Default action if no archive config is upload raw video.
    url = upload_file(path, api)
    video_def = make_video_definition(path)

    # If video was encoded, set codec_mime to video/mp4; otherwise do
    # not set codec_mime.
    if encoded:
        video_def['codec_mime'] = 'video/mp4'

    # Move video file with the api.
    response = api.move_video(media, move_video_spec={
        'media_files': {'archival': [{
            **video_def,
            'url': url,
        }]},
    })
    assert isinstance(response, CreateResponse)

def convert_archival(host, token, media, path, outpath, raw_width, raw_height):
    # Retrieve this media's type to inspect archive config.
    api = get_api(host, token)
    media_obj = api.get_media(media)
    media_type = api.get_media_type(media_obj.meta)

    if media_type.archive_config is None:
        default_archival_upload(api, host, media, path, False)
    else:
        for idx, archive_config in enumerate(media_type.archive_config):
            if archive_config.encode is None:
                # If no encode, just use the original file.
                output_file = path
            else:
                # Encode the media to archival format.
                os.makedirs(outpath, exist_ok=True)
                output_file = os.path.join(outpath, f"archival_{idx}.mp4")
                codec = find_best_encoder(archive_config.encode.vcodec)
                quality_flag = "-crf"
                pixel_format = "yuv420p"
                tune_settings = ["-preset", archive_config.encode.preset,
                                 "-tune", archive_config.encode.tune]
                if codec.find("qsv") >= 0:
                    quality_flag = "-global_quality"
                    pixel_format = "nv12"
                    tune_settings=[] #QSV doesn't do tuning
                elif codec == "libsvt_hevc":
                    # SVT for HEVC does not do tuning or CRF
                    tune_settings=[]
                    quality_flag = "-global_quality"
                cmd = [
                    "ffmpeg", "-y",
                    "-i", path,
                    "-vcodec", codec,
                    "-vf", "yadif",
                    quality_flag, str(archive_config.encode.crf),
                    "-pix_fmt", pixel_format,
                    *tune_settings
                ]
                if archive_config.encode.vcodec == 'hevc':
                    cmd += ["-tag:v", "hvc1"]
                elif archive_config.encode.vcodec == 'h264':
                    cmd += ["-tag:v", "avc1"]
                cmd.append(output_file)
                    
                logger.info('ffmpeg cmd = {}'.format(cmd))
                subprocess.run(cmd, check=True)

            if archive_config.s3_storage is None:
                default_archival_upload(api, host, media, output_file, True)
            else:
                import boto3
                # Get credentials from config object.
                aws_access_key = archive_config.s3_storage.aws_access_key
                aws_secret_access_key = archive_config.s3_storage.aws_secret_access_key
                bucket_name = archive_config.s3_storage.bucket_name
                logger.info(f"Uploading {output_file} to S3 bucket {bucket_name}...")

                # Upload the video to S3.
                client = boto3.client('s3', aws_access_key_id=aws_access_key,
                                      aws_secret_access_key=aws_secret_access_key)
                client.upload_file(output_file, bucket_name, os.path.basename(output_file))

def make_audio_definition(disk_file):
    cmd = [
        "ffprobe",
        "-v","error",
        "-show_entries", "stream",
        "-print_format", "json",
        "-select_streams", "a",
        disk_file,
    ]
    output = subprocess.run(cmd, stdout=subprocess.PIPE, check=True).stdout
    audio_info = json.loads(output)
    stream_idx=0
    for idx, stream in enumerate(audio_info["streams"]):
        if stream["codec_type"] == "audio":
            stream_idx=idx
            break
    stream = audio_info["streams"][stream_idx]
    audio_def = {"codec": stream["codec_name"],
                 "codec_description": stream["codec_long_name"],
                 "size": os.stat(disk_file).st_size,
                 "bit_rate": int(stream.get("bit_rate",-1))}
    return audio_def

def convert_audio(host, token, media, path, outpath):
    os.makedirs(outpath, exist_ok=True)
    logger.info("Extracting audio")
    output_file = os.path.join(outpath, f"audio.m4a")
    audio_extraction=["ffmpeg", "-y",
                      "-i", path,
                      "-vn", # Strip video
                      "-c:a", "aac",
                      "-ac", "2",
                      output_file]
    subprocess.run(audio_extraction, check=True)
    logger.info("Finished extracting audio!")
  
    # Upload audio. 
    api = get_api(host, token)
    url = upload_file(output_file, api)
   
    # Move video file with the api.
    response = api.move_video(media, move_video_spec={
        'media_files': {'audio': [{
            **make_audio_definition(output_file),
            'url': url,
        }]},
    })
    assert isinstance(response, CreateResponse)

def get_length_info(stream):
    """ Given a json dump of the stream return the length of the video """
    fps_fractional = stream["avg_frame_rate"].split("/")
    fps = float(fps_fractional[0]) / float(fps_fractional[1])

    start_time = float(stream["start_time"])
    if 'duration' in stream:
        seconds = float(stream["duration"])
    elif 'tags' in stream:
        if 'DURATION' in stream['tags']:
            length = stream['tags']['DURATION'].split(':')
            seconds = float(length[0])*3600
            seconds += float(length[1])*60
            seconds += float(length[2])
    else:
        raise Exception('No way to determine file length!')

    num_frames = float(fps * seconds)
    return fps,int(num_frames)

if __name__ == '__main__':
    args = parse_args()
    if args.category == 'streaming':
        if args.configs == '':
            configs = []
        else:
            configs = [res for res in args.configs.split(',')]
        convert_streaming(args.host, args.token, args.media, args.input, args.output,
                          args.raw_width, args.raw_height, configs)
    elif args.category == 'archival':
        convert_archival(args.host, args.token, args.media, args.input, args.output,
                         args.raw_width, args.raw_height)
    elif args.category == 'audio':
        convert_audio(args.host, args.token, args.media, args.input, args.output)



from uuid import uuid1
import mimetypes
import os
import math
import io
import tarfile

from tusclient.client import TusClient
from urllib.parse import urljoin
from urllib.parse import urlsplit

from .md5sum import md5sum

def upload_media_archive(api, project, paths, section="Test Section", chunk_size=2*1024*1024):
    """ Uploads multiple media files as an archive.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        for progress, response in tator.util.upload_media_archive(api, project_id, paths):
            print(f"Upload progress: {progress}%")
        print(response.message)

    :param api: :class:`tator.TatorApi` object.
    :param project: Unique integer identifying a project.
    :param paths: Path to a media archive or list of paths to media files.
    :param section: [Optional] Media section to upload to.
    :param chunk_size: [Optional] Chunk size in bytes. Default is 2MB.
    :returns: Generator that yields tuple containing progress (0-100) and a
        response. The response is `None` until the last yield, when the response
        is the response object from :meth:`tator.TatorApi.transcode`.
    """
    upload_uid = str(uuid1())
    upload_gid = str(uuid1())
    host = api.api_client.configuration.host
    token = api.api_client.configuration.api_key['Authorization']
    prefix = api.api_client.configuration.api_key_prefix['Authorization']
    tusURL = urljoin(host, "files/")
    tus = TusClient(tusURL, headers={'Authorization': f'{prefix} {token}',
                                     'Upload-Uid': f'{upload_uid}'})
    if isinstance(paths, list):
        in_mem_buf = io.BytesIO()
        in_mem_tar = tarfile.TarFile(mode='w', fileobj=in_mem_buf)
        for idx,fp in enumerate(paths):
            in_mem_tar.add(fp, os.path.basename(fp))

        uploader = tus.uploader(file_stream=in_mem_buf, chunk_size=chunk_size,
                                retries=10, retry_delay=15)
    else:
        file_buf = open(paths, 'rb')
        uploader = tus.uploader(file_stream=file_buf, chunk_size=chunk_size,
                                retries=10, retry_delay=15)
    last_progress = 0
    num_chunks=math.ceil(uploader.get_file_size()/chunk_size)
    yield (last_progress, None)
    for chunk_count in range(num_chunks):
        uploader.upload_chunk()
        this_progress = round((chunk_count / num_chunks) *100,1)
        if this_progress != last_progress:
            yield (this_progress, None)
            last_progress = this_progress

    # Initiate transcode.
    spec = {
        'type': -1, #Tar-based import
        'uid': upload_uid,
        'gid': upload_gid,
        'url': uploader.url,
        'name': "archive.tar",
        'section': section,
        'md5': "N/A",
    }
    response = api.transcode(project, transcode_spec=spec)
    yield (100, response)



import os
import math
import tempfile
from uuid import uuid1
from collections import defaultdict

from tusclient.client import TusClient
from urllib.parse import urljoin

from .md5sum import md5sum
from ._download_file import _download_file

class HostTransfer:
    def __init__(self, src_api, dest_api):
        """ Sets up authentication for transferring file.

        :param src_api: :class:`tator.TatorApi` object corresponding to source host.
        :param dest_api: :class:`tator.TatorApi` object corresponding to destination host.
        """
        # Set up source auth.
        config = src_api.api_client.configuration
        token = config.api_key['Authorization']
        prefix = config.api_key_prefix['Authorization']
        self.src_host = config.host
        self.src_headers = {
            'Authorization': f'{prefix} {token}',
            'Content-Type': f'application/json',
            'Accept-Encoding': 'gzip',
        }

        # Set up destination auth.
        config = dest_api.api_client.configuration
        self.dest_token = config.api_key['Authorization']
        self.dest_prefix = config.api_key_prefix['Authorization']
        self.dest_host = config.host
        self.tus_url = urljoin(self.dest_host, 'files/')

    def __call__(self, src_url):
        """ Downloads a file from one host and uploads it to another.
        
        :param src_url: Relative URL of source file to download.
        :returns: URL of destination file.
        """
        CHUNK_SIZE = 5*1024*1024
        url = urljoin(self.src_host, src_url)
        upload_uid = str(uuid1())
        tus_headers = {'Authorization': f'{self.dest_prefix} {self.dest_token}',
                       'Upload-Uid': f'{upload_uid}'}
        tus = TusClient(self.tus_url, headers=tus_headers)
        with tempfile.NamedTemporaryFile() as temp:
            for _ in _download_file(url, self.src_headers, temp.name):
                pass
            uploader = tus.uploader(temp.name, chunk_size=CHUNK_SIZE,
                                    retries=10, retry_delay=15)
            num_chunks = math.ceil(uploader.get_file_size()/CHUNK_SIZE)
            for chunk_count in range(num_chunks):
                uploader.upload_chunk()
        return uploader.url

def clone_media_list(src_api, query_params, dest_project, dest_type=-1, dest_section='', 
                     dest_api=None):
    """ Clone media list.

    This can be used to clone media from one project to another or from one
    host to another. In the case of the same host, the media files are not
    copied. If the destination host is different (`dest_api` is a :class:`tator.TatorApi`
    object), the transcoded files for each media object is downloaded and then
    uploaded to the other host, and a new :class:`tator.Media` object is created.

    Example for different host:

    .. code-block:: python

        src_api = tator.get_api(host, token)
        dest_api = tator.get_api(other_host, other_token)
        query_params = {'project': 1, 'media_id': [1]}
        dest_project = 1
        dest_type = -1
        dest_section = 'My cloned media'
        created_ids = []
        generator = clone_media_list(src_api, query_params, dest_project, dest_type,
                                     dest_section, dest_api)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} files...")
            created_ids.append(response.id)
        print(f"Finished creating {num_created} files!")

    Example for same host:

    .. code-block:: python

        api = tator.get_api(host, token)
        query_params = {'media_id': [1]}
        dest_project = 1
        created_ids = []
        generator = clone_media_list(src_api, query_params, dest_project)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} files...")
            created_ids += response.id # This response is from the CloneMedia endpoint.
        print(f"Finished creating {num_created} files!")

    :param src_api: :class:`tator.TatorApi` object corresponding to source host or only
        host if this is a clone on same host.
    :param query_params: Dictionary containing query parameters for source media list.
    :param dest_project: Unique integer identifying destination project.
    :param dest_type: Unique integer identifying destination media type. If set to
        -1, the media type is set to the first media type in the project.
    :param dest_section: Name of destination section.
    :param dest_api: :class:`tator.TatorApi` object corresponding to destination host.
    :returns: Generator containing number of files created, number of files total,
        most recent response from media creation operation, and mapping between original IDs
        and created IDs.
    """
    # Make sure query has a project.
    if 'project' not in query_params:
        raise Exception("Query parameters must include a project!")

    # Guess the media type if it was not given.
    if dest_type == -1:
        if dest_api is None:
            media_types = src_api.get_media_type_list(dest_project)
        else:
            media_types = dest_api.get_media_type_list(dest_project)
        if len(media_types) == 0:
            raise Exception('Specified project does not have any media types!')
        dest_type = media_types[0].id

    # Get medias.
    medias = src_api.get_media_list(**query_params)
    total_files = len(medias)

    # Clone the media.
    created_ids = []
    if (dest_api is None) or (dest_api is src_api):
        for idx in range(0, len(medias), 100): # Go in batches of 100
            # Clone media locally.
            media_ids = [media.id for media in medias[idx:idx+100]]
            response = src_api.clone_media_list(query_params['project'],
                                                media_id=media_ids,
                                                clone_media_spec={
                                                    'dest_project': dest_project,
                                                    'dest_section': dest_section,
                                                    'dest_type': dest_type,
                                                })
            created_ids += response.id
            id_map = {src_id: dest_id for src_id, dest_id in zip(media_ids, response.id)}
            yield (len(created_ids), total_files, response, id_map)
    else:
        # Clone media to another host.
        transfer = HostTransfer(src_api, dest_api)
        for media in medias:
            # Set up media spec.
            attributes = media.attributes
            if 'tator_user_sections' in attributes:
                attributes.pop('tator_user_sections', None)
            media_spec = {
                'type': dest_type,
                'name': media.name,
                'gid': media.gid,
                'uid': media.uid,
                'md5': media.md5,
                'fps': media.fps,
                'num_frames': media.num_frames,
                'codec': media.codec,
                'width': media.width,
                'height': media.height,
                'attributes': attributes,
                'section': dest_section if dest_section else media.section,
            }
            # Transfer thumbnails and image.
            media_spec['thumbnail_url'] = transfer(f'/media/{media.thumbnail}')

            if media.thumbnail_gif:
                media_spec['thumbnail_gif_url'] = transfer(f'/media/{media.thumbnail_gif}')

            if media.file:
                media_spec['url'] = transfer(f'/media/{media.file}')

            # Create the media object.
            response = dest_api.create_media(dest_project, media_spec=media_spec)
            id_map = {media.id: response.id}

            # Transfer videos.
            if media.media_files:
                if media.media_files.archival:
                    for archival in media.media_files.archival:
                        media_def = {k: v for k, v in archival.to_dict().items()
                                     if v is not None}
                        media_def.pop('path', None)
                        media_def['url'] = transfer(archival.path)
                        dest_api.move_video(response.id, move_video_spec={
                            'media_files': {'archival': [media_def]}
                        })
                if media.media_files.streaming:
                    for streaming in media.media_files.streaming:
                        media_def = {k: v for k, v in streaming.to_dict().items()
                                     if v is not None}
                        media_def.pop('path', None)
                        media_def['url'] = transfer(streaming.path)
                        media_def['segments_url'] = transfer(streaming.segment_info)
                        dest_api.move_video(response.id, move_video_spec={
                            'media_files': {'streaming': [media_def]}
                        })
                if media.media_files.audio:
                    for audio in media.media_files.audio:
                        media_def = {k: v for k, v in audio.to_dict().items()
                                     if v is not None}
                        media_def.pop('path', None)
                        media_def['url'] = transfer(audio.path)
                        dest_api.move_video(response.id, move_video_spec={
                            'media_files': {'audio': [media_def]}
                        })
            created_ids.append(response.id)
            yield (len(created_ids), total_files, response, id_map)



def _convert_for_post(state, version_mapping, media_mapping, localization_mapping,
                      state_type_mapping):
    # Swap version IDs.
    version_id = state.version
    if version_id in version_mapping:
        version_id = version_mapping[version_id]
    else:
        raise ValueError(f"Source version ID {version_id} missing from version_mapping!")
    # Swap media IDs.
    media_ids = state.media
    for idx, media_id in enumerate(media_ids):
        if media_id in media_mapping:
            media_ids[idx] = media_mapping[media_id]
        else:
            raise ValueError(f"Source media ID {media_id} missing from media_mapping!")
    # Swap localization IDs.
    localization_ids = state.localizations
    for idx, localization_id in enumerate(localization_ids):
        if localization_id in localization_mapping:
            localization_ids[idx] = localization_mapping[localization_id]
        else:
            raise ValueError(f"Source localization ID {localization_id} missing from "
                              "localization_mapping!")
    # Swap state type IDs.
    state_type_id = state.meta
    if state_type_id in state_type_mapping:
        state_type_id = state_type_mapping[state_type_id]
    else:
        raise ValueError(f"Source state_type ID {state_type_id} missing from "
                          "state_type_mapping!")
    # Fill in required fields for post.
    spec = {'type': state_type_id,
            'version': version_id,
            'media_ids': media_ids,
            'localization_ids': localization_ids,
            **state.attributes}
    if state.frame is not None:
        spec['frame'] = state.frame
    return spec

def clone_state_list(src_api, query_params, dest_project, version_mapping, media_mapping,
                     localization_mapping, state_type_mapping, dest_api=None):
    """ Clone state list.

    This can be used to clone states from one project to another or from one
    host to another.

    Example for different host:

    .. code-block:: python

        src_api = tator.get_api(host, token)
        dest_api = tator.get_api(other_host, other_token)
        query_params = {'project': 1, 'media_id': [1]}
        version_mapping = {1: 10} # Source version ID -> Dest version ID
        media_mapping = {1: 10} # Source media ID -> dest media ID
        localization_mapping = {} # For tracks, source localization ID -> dest localization ID
        state_type_mapping = {1: 10} # Source state type ID -> Dest state type ID
        dest_project = 1
        dest_version = -1
        created_ids = []
        generator = clone_state_list(src_api, query_params, dest_project, version_mapping,
                                     media_mapping, localization_mapping, state_type_mapping,
                                     dest_api)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} states...")
            created_ids.append(response.id)
        print(f"Finished creating {num_created} states!")

    Example for same host:

    .. code-block:: python

        api = tator.get_api(host, token)
        query_params = {'media_id': [1]}
        dest_project = 1
        version_mapping = {1: 10} # Source version ID -> Dest version ID
        media_mapping = {1: 10}
        localization_mapping = {}
        state_type_mapping = {1: 10} # Source state type ID -> Dest state type ID
        created_ids = []
        generator = clone_state_list(src_api, query_params, dest_project, version_mapping,
                                     media_mapping, localization_mapping, state_type_mapping)
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} states...")
            created_ids += response.id
        print(f"Finished creating {num_created} states!")

    :param src_api: :class:`tator.TatorApi` object corresponding to source host or only
        host if this is a clone on same host.
    :param query_params: Dictionary containing query parameters for source state list.
    :param dest_project: Unique integer identifying destination project.
    :param version_mapping: Dictionary mapping source version IDs to destination version IDs. If the
        source localization list contains a version ID for which a destination version is not supplied, 
        an exception is raised.
    :param media_mapping: Dictionary mapping source media IDs to destination media IDs. If the
        source state list contains a media ID for which a destination media is not supplied, 
        an exception is raised.
    :param localization_mapping: Dictionary mapping source localization IDs to destination 
        localization IDs. If the source state list contains a localization ID for which a
        destination localization ID is not supplied, an exception is raised.
    :param state_type_mapping: Dictionary mapping source state type IDs to destination 
        state_type IDs. If the source state list contains a state type ID for which a
        destination state type ID is not supplied, an exception is raised.
    :param dest_api: :class:`tator.TatorApi` object corresponding to destination host.
    :returns: Generator containing number of states created, number of states total,
        most recent response from state creation operation, and mapping between
        original IDs and created IDs.
    """
    # Make sure query has a project.
    if 'project' not in query_params:
        raise Exception("Query parameters must include a project!")

    # Set the dest_api if not given.
    if dest_api is None:
        dest_api = src_api

    # Start by getting list of states to be cloned.
    states = src_api.get_state_list(**query_params)

    # Convert to new spec.
    spec = [_convert_for_post(state, version_mapping, media_mapping, localization_mapping,
                              state_type_mapping) for state in states]

    # Create the states.
    created_ids = []
    total_states = len(spec)
    for idx in range(0, total_states, 500):
        response = dest_api.create_state_list(dest_project,
                                              state_spec=spec[idx:idx+500])
        created_ids += response.id
        id_map = {src.id: dest_id for src, dest_id in zip(states[idx:idx+500], response.id)}
        yield (len(created_ids), total_states, response, id_map)




#!/usr/bin/env python

import subprocess
import json
import argparse
import logging
import os

from ..openapi.tator_openapi.models import MediaType
from ..util.get_api import get_api
from ..util.get_parser import get_parser
from .transcode import get_length_info
from collections import defaultdict

STREAMING_RESOLUTIONS=[144, 360, 480, 720, 1080]

logger = logging.getLogger(__name__)

def parse_args():
    parser = get_parser()
    parser.add_argument('path', help='Path to original file.')
    parser.add_argument('--project', type=int, help='Unique integer identifying a project. This is '
                                                    'only needed if media_type is -1.')
    parser.add_argument('--media_type', type=int, help='Unique integer identifying a media type.')
    parser.add_argument('--group_to', type=int, default=480,
                         help='Vertical resolutions below this will be transcoded with '
                              'multi-headed ffmpeg.')
    parser.add_argument('--output', help='Path to output json file.')
    return parser.parse_args()

def determine_transcode(host, token, media_type, path, group_to):
    """ Determine transcode workloads to be performed.

    :param host: URL of host.
    :param token: API token.
    :param media_type: Unique integer identifying a media type.
    :param path: Path to original file.
    :param group_to: Resolutions less or equal to this will be grouped into one workload.
    """
    cmd = [
        "ffprobe",
        "-v","error",
        "-show_entries", "stream",
        "-print_format", "json",
        "-count_frames",
        "-skip_frame", "nokey",
        path,
    ]
    output = subprocess.run(cmd, stdout=subprocess.PIPE, check=True).stdout
    video_info = json.loads(output)
    stream_idx=0
    audio=False
    for idx, stream in enumerate(video_info["streams"]):
        if stream["codec_type"] == "video":
            stream_idx=idx
        if stream["codec_type"] == "audio":
            logger.info("Found Audio Track")
            audio=True
    stream = video_info["streams"][stream_idx]
    fps, num_frames = get_length_info(stream)

    # Handle up to but not exceeding FHD
    height = int(stream["height"])
    width = int(stream["width"])

    # Handle rotated videos
    if "tags" in stream:
        if "rotate" in stream["tags"]:
            if stream["tags"]["rotate"] == "90":
                height = int(stream["width"])
                width = int(stream["height"])
    print(f"Height of video is : {height}")

    # Generate output path
    base, ext = os.path.splitext(path)

    # Get media type object.
    api = get_api(host, token)
    if media_type == -1:
        media_types = api.get_media_type_list(args.project)
        for media_type_obj in media_types:
            if media_type_obj.dtype == 'video':
                break
    else:
        media_type_obj = api.get_media_type(media_type)
    assert isinstance(media_type_obj, MediaType)

    available_resolutions = STREAMING_RESOLUTIONS
    crf_map = defaultdict(lambda: 23)
    codec_map = defaultdict(lambda: 'libx264')
    try:
        if media_type_obj.streaming_config:
            available_resolutions = []
            for config in media_type_obj.streaming_config:
                available_resolutions.append(config.resolution)
                crf_map[config.resolution] = config.crf
                codec_map[config.resolution] = config.vcodec
    except Exception as e:
        # Likely an old version of the server
        # TODO: Remove me someday
        print(e)
        print("Defaulting to STREAMING_RESOLUTIONS")
    print(f"Selected Resolutions {available_resolutions}")
    # Make a list of resolutions needed
    resolutions = [resolution for resolution in available_resolutions if resolution < height]
    if height <= max(available_resolutions) and not height in resolutions:
        resolutions.append(height)

    # Streaming workloads (low res)
    workloads = [{
        'category': 'streaming',
        'path': path,
        'raw_height': height,
        'raw_width': width,
        'configs': [f"{resolution}:{crf_map[resolution]}:{codec_map[resolution]}"
                    for resolution in resolutions if resolution <= group_to],
    }]

    # Streaming workloads (higher res)
    workloads += [{
        'category': 'streaming',
        'path': path,
        'raw_height': height,
        'raw_width': width,
        'configs': [f"{resolution}:{crf_map[resolution]}:{codec_map[resolution]}"],
    } for resolution in resolutions if resolution > group_to]

    # Archival workloads
    # This will force all transcodes to run `convert_archival` on the raw video,
    # which will fetch the media type and take action based on the returned 
    # archive config.
    workloads += [{
        'category': 'archival',
        'path': path,
        'raw_height': height,
        'raw_width': width,
        'configs': [],
    }]

    # Audio workloads
    if audio:
        workloads += [{
            'category': 'audio',
            'path': path,
            'raw_height': height,
            'raw_width': width,
            'configs': [],
        }]

    return workloads

if __name__ == '__main__':
    args = parse_args()
    workloads = determine_transcode(args.host, args.token, args.media_type, args.path,
                                    args.group_to)
    for workload in workloads:
        workload['configs'] = ','.join(workload['configs'])
    with open(args.output, 'w') as f:
        json.dump(workloads, f)



import datetime
import random
import string
import uuid
import time

import tator
from ._common import assert_close_enough

def random_leaf(project, leaf_type, parent_obj=None, post=False):
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
    }
    name = "".join(random.choice(string.ascii_letters) for _ in range(10))
    out = {
        'project': project,
        'type': leaf_type,
        'name': name,
    }
    if parent_obj:
        out['parent'] = parent_obj.id
    if post:
        out = {**out, **attributes}
    else:
        out['attributes'] = attributes
    return out

def test_leaf_crud(host, token, project, clone_project, leaf_type, clone_leaf_type):
    tator_api = tator.get_api(host, token)

    # Create root leaf.
    root_spec = random_leaf(project, leaf_type, None, True)
    response = tator_api.create_leaf_list(project=project, leaf_spec=[root_spec])
    assert(isinstance(response, tator.models.CreateListResponse))
    prev_ids = response.id

    time.sleep(5)

    # Create three additional layers.
    for _ in range(3):
        new_ids = []
        for parent_id in prev_ids:
            parent_obj = tator_api.get_leaf(parent_id)
            leaf_spec = [random_leaf(project, leaf_type, parent_obj, True)
                         for _ in range(3)] # 3 children per parent
            response = tator_api.create_leaf_list(project=project, leaf_spec=leaf_spec)
            assert(isinstance(response, tator.models.CreateListResponse))
            new_ids += response.id
        prev_ids = list(new_ids)

    time.sleep(1)

    # Clone the layers.
    parent_mapping = {}
    for depth in range(4):
        leaves = tator_api.get_leaf_list(project=project, depth=depth)
        generator = tator.util.clone_leaf_list(tator_api, {'project': project, 'depth': depth},
                                               clone_project, parent_mapping,
                                               {leaf_type: leaf_type})
        created_ids = []
        for num_created, num_total, response, id_map in generator:
            print(f"Created {num_created} of {num_total} leafs...")
            created_ids += response.id
        parent_mapping = {**parent_mapping,
                          **{src.id: dst for src, dst in zip(leaves, created_ids)}}
        print(f"Finished creating {num_created} leafs!")
    for src_id in parent_mapping:
        src = tator_api.get_leaf(src_id)
        dst = tator_api.get_leaf(parent_mapping[src_id])
        assert(src.path.split('.', 1)[1] == dst.path.split('.', 1)[1])



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Project(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'int',
        'name': 'str',
        'num_files': 'int',
        'permission': 'str',
        'size': 'int',
        'summary': 'str',
        'thumb': 'str',
        'usernames': 'list[str]'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'num_files': 'num_files',
        'permission': 'permission',
        'size': 'size',
        'summary': 'summary',
        'thumb': 'thumb',
        'usernames': 'usernames'
    }

    def __init__(self, id=None, name=None, num_files=None, permission=None, size=None, summary='', thumb=None, usernames=None, local_vars_configuration=None):  # noqa: E501
        """Project - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._num_files = None
        self._permission = None
        self._size = None
        self._summary = None
        self._thumb = None
        self._usernames = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if num_files is not None:
            self.num_files = num_files
        if permission is not None:
            self.permission = permission
        if size is not None:
            self.size = size
        if summary is not None:
            self.summary = summary
        if thumb is not None:
            self.thumb = thumb
        if usernames is not None:
            self.usernames = usernames

    @property
    def id(self):
        """
        Unique integer identifying the project.

        :return: The id of this Project. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying the project.

        :param id: The id of this Project.
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """
        Name of the project.

        :return: The name of this Project. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the project.

        :param name: The name of this Project.
        :type: str
        """

        self._name = name

    @property
    def num_files(self):
        """
        Number of files in the project.

        :return: The num_files of this Project. 
        :rtype: int
        """
        return self._num_files

    @num_files.setter
    def num_files(self, num_files):
        """
        Number of files in the project.

        :param num_files: The num_files of this Project.
        :type: int
        """

        self._num_files = num_files

    @property
    def permission(self):
        """
        Permission level of user making request.

        :return: The permission of this Project. 
        :rtype: str
        """
        return self._permission

    @permission.setter
    def permission(self, permission):
        """
        Permission level of user making request.

        :param permission: The permission of this Project.
        :type: str
        """

        self._permission = permission

    @property
    def size(self):
        """
        Size of the project in bytes.

        :return: The size of this Project. 
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """
        Size of the project in bytes.

        :param size: The size of this Project.
        :type: int
        """

        self._size = size

    @property
    def summary(self):
        """
        Summary of the project.

        :return: The summary of this Project. 
        :rtype: str
        """
        return self._summary

    @summary.setter
    def summary(self, summary):
        """
        Summary of the project.

        :param summary: The summary of this Project.
        :type: str
        """

        self._summary = summary

    @property
    def thumb(self):
        """
        URL of thumbnail used to represent the project.

        :return: The thumb of this Project. 
        :rtype: str
        """
        return self._thumb

    @thumb.setter
    def thumb(self, thumb):
        """
        URL of thumbnail used to represent the project.

        :param thumb: The thumb of this Project.
        :type: str
        """

        self._thumb = thumb

    @property
    def usernames(self):
        """
        List of usernames of project members.

        :return: The usernames of this Project. 
        :rtype: list[str]
        """
        return self._usernames

    @usernames.setter
    def usernames(self, usernames):
        """
        List of usernames of project members.

        :param usernames: The usernames of this Project.
        :type: list[str]
        """

        self._usernames = usernames

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Project):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Project):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class State(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'dict(str, object)',
        'frame': 'int',
        'id': 'int',
        'localizations': 'list[int]',
        'media': 'list[int]',
        'meta': 'int',
        'version': 'int'
    }

    attribute_map = {
        'attributes': 'attributes',
        'frame': 'frame',
        'id': 'id',
        'localizations': 'localizations',
        'media': 'media',
        'meta': 'meta',
        'version': 'version'
    }

    def __init__(self, attributes=None, frame=None, id=None, localizations=None, media=None, meta=None, version=None, local_vars_configuration=None):  # noqa: E501
        """State - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._frame = None
        self._id = None
        self._localizations = None
        self._media = None
        self._meta = None
        self._version = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        if frame is not None:
            self.frame = frame
        if id is not None:
            self.id = id
        if localizations is not None:
            self.localizations = localizations
        if media is not None:
            self.media = media
        if meta is not None:
            self.meta = meta
        if version is not None:
            self.version = version

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this State. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this State.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def frame(self):
        """
        Frame number this state applies to.

        :return: The frame of this State. 
        :rtype: int
        """
        return self._frame

    @frame.setter
    def frame(self, frame):
        """
        Frame number this state applies to.

        :param frame: The frame of this State.
        :type: int
        """

        self._frame = frame

    @property
    def id(self):
        """
        Unique integer identifying the state.

        :return: The id of this State. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying the state.

        :param id: The id of this State.
        :type: int
        """

        self._id = id

    @property
    def localizations(self):
        """
        List of localization IDs that this state applies to.

        :return: The localizations of this State. 
        :rtype: list[int]
        """
        return self._localizations

    @localizations.setter
    def localizations(self, localizations):
        """
        List of localization IDs that this state applies to.

        :param localizations: The localizations of this State.
        :type: list[int]
        """

        self._localizations = localizations

    @property
    def media(self):
        """
        List of media IDs that this state applies to.

        :return: The media of this State. 
        :rtype: list[int]
        """
        return self._media

    @media.setter
    def media(self, media):
        """
        List of media IDs that this state applies to.

        :param media: The media of this State.
        :type: list[int]
        """

        self._media = media

    @property
    def meta(self):
        """
        Unique integer identifying the entity type.

        :return: The meta of this State. 
        :rtype: int
        """
        return self._meta

    @meta.setter
    def meta(self, meta):
        """
        Unique integer identifying the entity type.

        :param meta: The meta of this State.
        :type: int
        """

        self._meta = meta

    @property
    def version(self):
        """
        Unique integer identifying the version.

        :return: The version of this State. 
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """
        Unique integer identifying the version.

        :param version: The version of this State.
        :type: int
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, State):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, State):
            return True

        return self.to_dict() != other.to_dict()



import time
import subprocess

import tator

def test_local_transcode(host, token, project, video_type, video_file):
    cmd = [
        'python3', '-m', 'tator.transcode', video_file,
        '--host', host,
        '--token', token,
        '--project', str(project),
        '--type', str(video_type),
        '--section', 'Locally transcoded',
    ]
    subprocess.run(cmd, check=True)

def test_bad_file(host, token, project, video_type, image_file):
    failed = False
    try:
        cmd = [
            'python3', '-m', 'tator.transcode', image_file,
            '--host', host,
            '--token', token,
            '--project', str(project),
            '--type', str(video_type),
            '--section', 'Bad transcodes',
        ]
        subprocess.run(cmd, check=True)
    except subprocess.CalledProcessError as cpe:
        failed = True
    assert(failed)
    time.sleep(2)
    # Make sure media file is gone.
    api = tator.get_api(host, token)
    medias = api.get_media_list(project, attribute='tator_user_sections::Bad transcodes')
    assert(len(medias) == 0)



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Media(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'dict(str, object)',
        'codec': 'str',
        'created_by': 'int',
        'created_datetime': 'str',
        'file': 'str',
        'fps': 'int',
        'gid': 'str',
        'height': 'int',
        'id': 'int',
        'last_edit_end': 'datetime',
        'last_edit_start': 'datetime',
        'md5': 'str',
        'media_files': 'MediaFiles',
        'meta': 'int',
        'modified_by': 'int',
        'modified_datetime': 'str',
        'name': 'str',
        'num_frames': 'int',
        'original': 'str',
        'project': 'int',
        'segment_info': 'str',
        'thumbnail': 'str',
        'thumbnail_gif': 'str',
        'uid': 'str',
        'width': 'int'
    }

    attribute_map = {
        'attributes': 'attributes',
        'codec': 'codec',
        'created_by': 'created_by',
        'created_datetime': 'created_datetime',
        'file': 'file',
        'fps': 'fps',
        'gid': 'gid',
        'height': 'height',
        'id': 'id',
        'last_edit_end': 'last_edit_end',
        'last_edit_start': 'last_edit_start',
        'md5': 'md5',
        'media_files': 'media_files',
        'meta': 'meta',
        'modified_by': 'modified_by',
        'modified_datetime': 'modified_datetime',
        'name': 'name',
        'num_frames': 'num_frames',
        'original': 'original',
        'project': 'project',
        'segment_info': 'segment_info',
        'thumbnail': 'thumbnail',
        'thumbnail_gif': 'thumbnail_gif',
        'uid': 'uid',
        'width': 'width'
    }

    def __init__(self, attributes=None, codec=None, created_by=None, created_datetime=None, file=None, fps=None, gid=None, height=None, id=None, last_edit_end=None, last_edit_start=None, md5=None, media_files=None, meta=None, modified_by=None, modified_datetime=None, name=None, num_frames=None, original=None, project=None, segment_info=None, thumbnail=None, thumbnail_gif=None, uid=None, width=None, local_vars_configuration=None):  # noqa: E501
        """Media - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._codec = None
        self._created_by = None
        self._created_datetime = None
        self._file = None
        self._fps = None
        self._gid = None
        self._height = None
        self._id = None
        self._last_edit_end = None
        self._last_edit_start = None
        self._md5 = None
        self._media_files = None
        self._meta = None
        self._modified_by = None
        self._modified_datetime = None
        self._name = None
        self._num_frames = None
        self._original = None
        self._project = None
        self._segment_info = None
        self._thumbnail = None
        self._thumbnail_gif = None
        self._uid = None
        self._width = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        if codec is not None:
            self.codec = codec
        if created_by is not None:
            self.created_by = created_by
        if created_datetime is not None:
            self.created_datetime = created_datetime
        if file is not None:
            self.file = file
        if fps is not None:
            self.fps = fps
        if gid is not None:
            self.gid = gid
        if height is not None:
            self.height = height
        if id is not None:
            self.id = id
        if last_edit_end is not None:
            self.last_edit_end = last_edit_end
        if last_edit_start is not None:
            self.last_edit_start = last_edit_start
        if md5 is not None:
            self.md5 = md5
        if media_files is not None:
            self.media_files = media_files
        if meta is not None:
            self.meta = meta
        if modified_by is not None:
            self.modified_by = modified_by
        if modified_datetime is not None:
            self.modified_datetime = modified_datetime
        if name is not None:
            self.name = name
        if num_frames is not None:
            self.num_frames = num_frames
        if original is not None:
            self.original = original
        if project is not None:
            self.project = project
        if segment_info is not None:
            self.segment_info = segment_info
        if thumbnail is not None:
            self.thumbnail = thumbnail
        if thumbnail_gif is not None:
            self.thumbnail_gif = thumbnail_gif
        if uid is not None:
            self.uid = uid
        if width is not None:
            self.width = width

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this Media. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this Media.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def codec(self):
        """
        Codec for videos.

        :return: The codec of this Media. 
        :rtype: str
        """
        return self._codec

    @codec.setter
    def codec(self, codec):
        """
        Codec for videos.

        :param codec: The codec of this Media.
        :type: str
        """

        self._codec = codec

    @property
    def created_by(self):
        """
        Unique integer identifying user who created this media.

        :return: The created_by of this Media. 
        :rtype: int
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """
        Unique integer identifying user who created this media.

        :param created_by: The created_by of this Media.
        :type: int
        """

        self._created_by = created_by

    @property
    def created_datetime(self):
        """
        Datetime when this media was created.

        :return: The created_datetime of this Media. 
        :rtype: str
        """
        return self._created_datetime

    @created_datetime.setter
    def created_datetime(self, created_datetime):
        """
        Datetime when this media was created.

        :param created_datetime: The created_datetime of this Media.
        :type: str
        """

        self._created_datetime = created_datetime

    @property
    def file(self):
        """
        URL of the media file. Relative to https://<domain>/media/.

        :return: The file of this Media. 
        :rtype: str
        """
        return self._file

    @file.setter
    def file(self, file):
        """
        URL of the media file. Relative to https://<domain>/media/.

        :param file: The file of this Media.
        :type: str
        """

        self._file = file

    @property
    def fps(self):
        """
        Frame rate for videos.

        :return: The fps of this Media. 
        :rtype: int
        """
        return self._fps

    @fps.setter
    def fps(self, fps):
        """
        Frame rate for videos.

        :param fps: The fps of this Media.
        :type: int
        """

        self._fps = fps

    @property
    def gid(self):
        """
        Group ID for the upload group of this media.

        :return: The gid of this Media. 
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """
        Group ID for the upload group of this media.

        :param gid: The gid of this Media.
        :type: str
        """

        self._gid = gid

    @property
    def height(self):
        """
        Vertical resolution in pixels.

        :return: The height of this Media. 
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """
        Vertical resolution in pixels.

        :param height: The height of this Media.
        :type: int
        """

        self._height = height

    @property
    def id(self):
        """
        Unique integer identifying this media.

        :return: The id of this Media. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying this media.

        :param id: The id of this Media.
        :type: int
        """

        self._id = id

    @property
    def last_edit_end(self):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :return: The last_edit_end of this Media. 
        :rtype: datetime
        """
        return self._last_edit_end

    @last_edit_end.setter
    def last_edit_end(self, last_edit_end):
        """
        Datetime of the end of the session when this media or its annotations were last edited.

        :param last_edit_end: The last_edit_end of this Media.
        :type: datetime
        """

        self._last_edit_end = last_edit_end

    @property
    def last_edit_start(self):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :return: The last_edit_start of this Media. 
        :rtype: datetime
        """
        return self._last_edit_start

    @last_edit_start.setter
    def last_edit_start(self, last_edit_start):
        """
        Datetime of the start of the session when this media or its annotations were last edited.

        :param last_edit_start: The last_edit_start of this Media.
        :type: datetime
        """

        self._last_edit_start = last_edit_start

    @property
    def md5(self):
        """
        MD5 checksum of the media file.

        :return: The md5 of this Media. 
        :rtype: str
        """
        return self._md5

    @md5.setter
    def md5(self, md5):
        """
        MD5 checksum of the media file.

        :param md5: The md5 of this Media.
        :type: str
        """

        self._md5 = md5

    @property
    def media_files(self):
        """

        :return: The media_files of this Media. 
        :rtype: MediaFiles
        """
        return self._media_files

    @media_files.setter
    def media_files(self, media_files):
        """

        :param media_files: The media_files of this Media.
        :type: MediaFiles
        """

        self._media_files = media_files

    @property
    def meta(self):
        """
        Unique integer identifying entity type of this media.

        :return: The meta of this Media. 
        :rtype: int
        """
        return self._meta

    @meta.setter
    def meta(self, meta):
        """
        Unique integer identifying entity type of this media.

        :param meta: The meta of this Media.
        :type: int
        """

        self._meta = meta

    @property
    def modified_by(self):
        """
        Unique integer identifying user who last modified this media.

        :return: The modified_by of this Media. 
        :rtype: int
        """
        return self._modified_by

    @modified_by.setter
    def modified_by(self, modified_by):
        """
        Unique integer identifying user who last modified this media.

        :param modified_by: The modified_by of this Media.
        :type: int
        """

        self._modified_by = modified_by

    @property
    def modified_datetime(self):
        """
        Datetime when this media was last modified.

        :return: The modified_datetime of this Media. 
        :rtype: str
        """
        return self._modified_datetime

    @modified_datetime.setter
    def modified_datetime(self, modified_datetime):
        """
        Datetime when this media was last modified.

        :param modified_datetime: The modified_datetime of this Media.
        :type: str
        """

        self._modified_datetime = modified_datetime

    @property
    def name(self):
        """
        Name of the media.

        :return: The name of this Media. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the media.

        :param name: The name of this Media.
        :type: str
        """

        self._name = name

    @property
    def num_frames(self):
        """
        Number of frames for videos.

        :return: The num_frames of this Media. 
        :rtype: int
        """
        return self._num_frames

    @num_frames.setter
    def num_frames(self, num_frames):
        """
        Number of frames for videos.

        :param num_frames: The num_frames of this Media.
        :type: int
        """

        self._num_frames = num_frames

    @property
    def original(self):
        """
        DEPRECATED. Use media_files. Stores path to original media file.

        :return: The original of this Media. 
        :rtype: str
        """
        return self._original

    @original.setter
    def original(self, original):
        """
        DEPRECATED. Use media_files. Stores path to original media file.

        :param original: The original of this Media.
        :type: str
        """

        self._original = original

    @property
    def project(self):
        """
        Unique integer identifying project of this media.

        :return: The project of this Media. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying project of this media.

        :param project: The project of this Media.
        :type: int
        """

        self._project = project

    @property
    def segment_info(self):
        """
        Path to segment info.

        :return: The segment_info of this Media. 
        :rtype: str
        """
        return self._segment_info

    @segment_info.setter
    def segment_info(self, segment_info):
        """
        Path to segment info.

        :param segment_info: The segment_info of this Media.
        :type: str
        """

        self._segment_info = segment_info

    @property
    def thumbnail(self):
        """
        URL of the thumbnail. Relative to https://<domain>/media/.

        :return: The thumbnail of this Media. 
        :rtype: str
        """
        return self._thumbnail

    @thumbnail.setter
    def thumbnail(self, thumbnail):
        """
        URL of the thumbnail. Relative to https://<domain>/media/.

        :param thumbnail: The thumbnail of this Media.
        :type: str
        """

        self._thumbnail = thumbnail

    @property
    def thumbnail_gif(self):
        """
        URL of the thumbnail gif for videos. Relative to https://<domain>/media/.

        :return: The thumbnail_gif of this Media. 
        :rtype: str
        """
        return self._thumbnail_gif

    @thumbnail_gif.setter
    def thumbnail_gif(self, thumbnail_gif):
        """
        URL of the thumbnail gif for videos. Relative to https://<domain>/media/.

        :param thumbnail_gif: The thumbnail_gif of this Media.
        :type: str
        """

        self._thumbnail_gif = thumbnail_gif

    @property
    def uid(self):
        """
        Unique ID for the upload of this media.

        :return: The uid of this Media. 
        :rtype: str
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """
        Unique ID for the upload of this media.

        :param uid: The uid of this Media.
        :type: str
        """

        self._uid = uid

    @property
    def width(self):
        """
        Horizontal resolution in pixels.

        :return: The width of this Media. 
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """
        Horizontal resolution in pixels.

        :param width: The width of this Media.
        :type: int
        """

        self._width = width

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Media):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Media):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class CloneMediaSpec(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'dest_project': 'int',
        'dest_section': 'str',
        'dest_type': 'int'
    }

    attribute_map = {
        'dest_project': 'dest_project',
        'dest_section': 'dest_section',
        'dest_type': 'dest_type'
    }

    def __init__(self, dest_project=None, dest_section=None, dest_type=None, local_vars_configuration=None):  # noqa: E501
        """CloneMediaSpec - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._dest_project = None
        self._dest_section = None
        self._dest_type = None
        self.discriminator = None

        self.dest_project = dest_project
        if dest_section is not None:
            self.dest_section = dest_section
        self.dest_type = dest_type

    @property
    def dest_project(self):
        """
        Unique integer identyifying destination project.

        :return: The dest_project of this CloneMediaSpec. 
        :rtype: int
        """
        return self._dest_project

    @dest_project.setter
    def dest_project(self, dest_project):
        """
        Unique integer identyifying destination project.

        :param dest_project: The dest_project of this CloneMediaSpec.
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and dest_project is None:  # noqa: E501
            raise ValueError("Invalid value for `dest_project`, must not be `None`")  # noqa: E501

        self._dest_project = dest_project

    @property
    def dest_section(self):
        """
        Destination media section name.

        :return: The dest_section of this CloneMediaSpec. 
        :rtype: str
        """
        return self._dest_section

    @dest_section.setter
    def dest_section(self, dest_section):
        """
        Destination media section name.

        :param dest_section: The dest_section of this CloneMediaSpec.
        :type: str
        """

        self._dest_section = dest_section

    @property
    def dest_type(self):
        """
        Unique integer identifying destination media type. Use -1 to automatically select the media type if only one media type exists in the destination project.

        :return: The dest_type of this CloneMediaSpec. 
        :rtype: int
        """
        return self._dest_type

    @dest_type.setter
    def dest_type(self, dest_type):
        """
        Unique integer identifying destination media type. Use -1 to automatically select the media type if only one media type exists in the destination project.

        :param dest_type: The dest_type of this CloneMediaSpec.
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and dest_type is None:  # noqa: E501
            raise ValueError("Invalid value for `dest_type`, must not be `None`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                dest_type is not None and dest_type < -1):  # noqa: E501
            raise ValueError("Invalid value for `dest_type`, must be a value greater than or equal to `-1`")  # noqa: E501

        self._dest_type = dest_type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CloneMediaSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CloneMediaSpec):
            return True

        return self.to_dict() != other.to_dict()



import os
from ..openapi import tator_openapi

def get_api(host='https://www.tatorapp.com', token=os.getenv('TATOR_TOKEN')):
    """ Retrieves a :class:`tator.api` instance using the given host and token.

    :param host: URL of host. Default is https://www.tatorapp.com.
    :param token: API token.
    :returns: :class:`tator.api` object.
    """
    config = tator_openapi.Configuration()
    config.host = host
    if token:
        config.api_key['Authorization'] = token
        config.api_key_prefix['Authorization'] = 'Token'
    return tator_openapi.TatorApi(tator_openapi.ApiClient(config))



import logging
import os
import shutil
import tempfile
import uuid
from textwrap import dedent

from tusclient.client import TusClient

import tator
from tator.transcode.upload import upload_file

logger = logging.getLogger(__name__)

def _create_yaml_file_str() -> str:
    """ Creates a argo manifest file used by the unit tests in this file
    """

    return dedent("""\
apiVersion: argoproj.io/v1alpha1
kind: Workflow
metadata:
  generateName: test-algorithm-launcher-
spec:
  entrypoint: pipeline
  ttlStrategy:
    secondsAfterCompletion: 300 # Time to live after workflow is completed, replaces ttlSecondsAfterFinished
    secondsAfterSuccess: 300 # Time to live after workflow is successful
    secondsAfterFailure: 600 # Time to live after workflow fails
  volumeClaimTemplates:
  - metadata:
      name: workdir
    spec:
      accessModes: [ "ReadWriteOnce" ]
      storageClassName: nfs-client
      resources:
        requests:
          storage: 100Mi
  templates:
  - name: pipeline
    steps:
    - - name: test
        template: test
  - name: test
    script:
      image: cvisionai/tator_transcoder #localhost:5000/tator_transcoder
      env:
      - name: TATOR_MEDIA_IDS
        value: "{{workflow.parameters.media_ids}}"
      - name: TATOR_API_SERVICE
        value: "{{workflow.parameters.rest_url}}"
      - name: TATOR_AUTH_TOKEN
        value: "{{workflow.parameters.rest_token}}"
      - name: TATOR_PROJECT_ID
        value: "{{workflow.parameters.project_id}}"
      - name: TATOR_WORK_DIR
        value: "/work"
      resources:
        limits:
          cpu: 1000m
          memory: 500Mi
      volumeMounts:
      - name: workdir
        mountPath: /work
      command: [python3]
      source: |
        #!/usr/bin/env python3

        import os
        import tator

        def main():
            host = os.path.dirname(os.getenv('TATOR_API_SERVICE'))
            token = os.getenv('TATOR_AUTH_TOKEN')
            project = int(os.getenv('TATOR_PROJECT_ID'))
            media_ids = os.getenv('TATOR_MEDIA_IDS')
            media_ids = [int(m) for m in media_ids.split(',')]

            tator_api = tator.get_api(host=host, token=token)

            print(f'{host} {token} {project} {media_ids}')

            for media in media_ids:
                name = f'test_media_{media}'
                dq = f'test_media_id:{media}'
                spec = tator.models.AnalysisSpec(name=name, data_query=dq)
                _ = tator_api.create_analysis(project=project, analysis_spec=spec)

        if __name__ == '__main__':
            main()
        """)

def _missing_upload_file(
        host: str,
        token: str,
        project: int) -> None:
    """ Test missing upload file for SaveAlgorithManifest endpoint

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
    """

    tator_api = tator.get_api(host=host, token=token)

    try:
        caught_exception = False
        spec = tator.models.AlgorithmManifestSpec(name='test.yaml', upload_url='not_there')
        tator_api.save_algorithm_manifest(project = project, algorithm_manifest_spec=spec)
    except:
        caught_exception = True

    assert caught_exception

def _upload_test_algorithm_manifest(
        host: str,
        token: str,
        project: int,
        manifest_name: str='test_manifest.yaml',
        break_yaml_file: bool=False) -> tator.models.AlgorithmManifest:
    """ Uploads the provided manifest file

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
        manifest_name: Local argo workflow manifest file to be uploaded and saved
        break_yaml_file: True if a syntax error is added to the given yaml.
            False leaves the yaml file untouched.

    Returns:
        Response from the save algorithm manifest endpoint
    """

    # Setup the interface to tator
    tator_api = tator.get_api(host=host, token=token)

    # Create the temporary manifest file
    fd, local_yaml_file = tempfile.mkstemp()
    try:
        with os.fdopen(fd, 'w') as file_handle:
            file_handle.write(_create_yaml_file_str())

            if break_yaml_file:
                file_handle.write('RANDOM_TEXT!!!!!')

        # Upload the manifest file with tus first
        logger.info(f"Created temporary manifest file: {local_yaml_file}")
        url = upload_file(path=local_yaml_file, api=tator_api)

        # Save the uploaded file using the save algorithm manifest endpoint
        spec = tator.models.AlgorithmManifestSpec(name=manifest_name, upload_url=url)
        response = tator_api.save_algorithm_manifest(project=project, algorithm_manifest_spec=spec)

    finally:
        os.remove(local_yaml_file)

    return response

def test_save_algorithm_manifest(
        host: str,
        token: str,
        project: int) -> None:
    """ Unit test for the SaveAlgorithmManifest endpoint

    Unit testing of the save algorithm endpoint involvest the following:
    - Provide something where the upload file doesn't exist
    - Upload two of the same files (have the same name, but will result in two separate files)

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
    """

    _missing_upload_file(host=host, token=token, project=project)

    response_1 = _upload_test_algorithm_manifest(
        host=host, token=token, project=project, manifest_name='test_nodupe.yaml')
    assert os.path.basename(response_1.url) == 'test_nodupe.yaml'

    response_2 = _upload_test_algorithm_manifest(
        host=host, token=token, project=project, manifest_name='test_nodupe.yaml')
    assert os.path.basename(response_2.url) == 'test_nodupe_0.yaml'

def test_register_algorithm(
        host: str,
        token: str,
        project: int) -> None:
    """ Unit test for the RegisterAlgorithm endpoint

    Unit testing of the algorithm registration endpoint involves the following:
    - Create a request body that's fine, but has bad syntax for the .yaml file
    - Create a request body for a .yaml file that doesn't exist
    - Normal request body

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
    """

    # Create a randomized unique algorithm name (that we'll end up deleting later anyway)
    ALGO_NAME = str(uuid.uuid1())

    # Get the user ID
    tator_api = tator.get_api(host=host, token=token)
    user = tator_api.whoami()
    user_id = user.id

    # Create a request body that's fine but has bad syntax for the .yaml file
    caught_exception = False
    try:
        response = _upload_test_algorithm_manifest(
            host=host, token=token, project=project, manifest_name='test.yaml', break_yaml_file=True)

        spec = tator.models.Algorithm(
            name=ALGO_NAME,
            project=project,
            user=user_id,
            description='test_description',
            manifest=response.url,
            cluster=None,
            files_per_job=1)

        response = tator_api.register_algorithm(project=project, algorithm_spec=spec)

    except:
        caught_exception = True

    assert caught_exception

    # Create a request body for a .yaml file that doesn't exist
    caught_exception = False
    try:
        spec = tator.models.Algorithm(
            name=ALGO_NAME,
            project=project,
            user=user_id,
            description='test_description',
            manifest='ghost',
            cluster=None,
            files_per_job=1)

        response = tator_api.register_algorithm(project=project, algorithm_spec=spec)

    except:
        caught_exception = True

    assert caught_exception

    # Create a normal algorithm workflow
    response = _upload_test_algorithm_manifest(
        host=host, token=token, project=project, manifest_name='test.yaml')

    manifest_url = response.url

    spec = tator.models.Algorithm(
        name=ALGO_NAME,
        project=project,
        user=user_id,
        description='test_description',
        manifest=manifest_url,
        cluster=None,
        files_per_job=1)

    response = tator_api.register_algorithm(project=project, algorithm_spec=spec)

    # Get the algorithm info
    algorithm_id = response.id
    algorithm_info = tator_api.get_algorithm(id=algorithm_id)
    spec.id = algorithm_id
    assert algorithm_info == spec

    # Attempt to patch the algorithm info and make sure it has been updated
    # Note: the cluster field is ignored in the patch operation
    spec = tator.models.Algorithm(
        name=str(uuid.uuid1()),
        project=project,
        user=user_id,
        description='new_test_description',
        manifest='coolfile.yaml',
        files_per_job=2)
    response = tator_api.update_algorithm(id=algorithm_id, algorithm_spec=spec)
    algorithm_info = tator_api.get_algorithm(id=algorithm_id)
    spec.id = algorithm_id
    assert algorithm_info == spec

    # Create another algorithm workflow and verify retrieving both algorithm objects
    # using the get list method
    algorithm_ids = [algorithm_id]

    new_spec = tator.models.Algorithm(
        name=str(uuid.uuid1()),
        project=project,
        user=user_id,
        description='test_description',
        manifest=manifest_url,
        cluster=None,
        files_per_job=1)

    response = tator_api.register_algorithm(project=project, algorithm_spec=new_spec)

    new_spec.id = response.id
    algorithm_ids.append(new_spec.id)
    spec_list = [spec, new_spec]
    algorithm_list = tator_api.get_algorithm_list(project=project)

    for alg in algorithm_list:
        found_match = False
        for spec in spec_list:
            if spec == alg:
                found_match = True
                break

        assert found_match

    # Finally delete all the algorithms
    current_number_of_algs = len(algorithm_ids)
    for alg_id in algorithm_ids:
        _ = tator_api.delete_algorithm(id=alg_id)
        algorithm_list = tator_api.get_algorithm_list(project=project)

        current_number_of_algs -= 1
        assert len(algorithm_list) == current_number_of_algs

def test_register_algorithm_with_missing_fields(
        host: str,
        token: str,
        project: int) -> None:
    """ Unit test for the RegisterAlgorithm endpoint focused on missing request body fields

    Request bodies are created with missing required fields and a workflow tries
    to be registered with these incorrect request bodies.

    Args:
        host: Project URL
        token: User token used for connecting to the host
        project: Unique identifier of test project
    """

    NAME = str(uuid.uuid1())
    DESCRIPTION = 'description'
    CLUSTER = 1
    FILES_PER_JOB = 1

    # Setup the interface to tator and get the user ID
    tator_api = tator.get_api(host=host, token=token)
    user = tator_api.whoami()
    user_id = user.id

    # Upload a manifest file
    response = _upload_test_algorithm_manifest(
        host=host, token=token, project=project, manifest_name='test.yaml')
    manifest_url = response.url

    # Missing name field
    caught_exception = False
    try:
        spec = tator.models.Algorithm(
            project=project,
            user=user_id,
            description=DESCRIPTION,
            manifest=manifest_url,
            cluster=CLUSTER,
            files_per_job=FILES_PER_JOB)

        response = tator_api.register_algorithm(project=project, algorithm_spec=spec)
    except:
        caught_exception = True

    assert caught_exception

    # Missing user field
    caught_exception = False
    try:
        spec = tator.models.Algorithm(
            name=NAME,
            project=project,
            description=DESCRIPTION,
            manifest=manifest_url,
            cluster=CLUSTER,
            files_per_job=FILES_PER_JOB)

        response = tator_api.register_algorithm(project=project, algorithm_spec=spec)
    except:
        caught_exception = True

    assert caught_exception

    # Missing description field
    caught_exception = False
    try:
        spec = tator.models.Algorithm(
            name=NAME,
            project=project,
            user=user_id,
            description=DESCRIPTION,
            manifest=manifest_url,
            cluster=CLUSTER,
            files_per_job=FILES_PER_JOB)

        response = tator_api.register_algorithm(project=project, algorithm_spec=spec)
    except:
        caught_exception = True

    assert caught_exception

    # Missing manifest
    caught_exception = False
    try:
        spec = tator.models.Algorithm(
            name=NAME,
            project=project,
            user=user_id,
            description=DESCRIPTION,
            cluster=CLUSTER,
            files_per_job=FILES_PER_JOB)

        response = tator_api.register_algorithm(project=project, algorithm_spec=spec)
    except:
        caught_exception = True

    assert caught_exception

    # Missing fields per job
    caught_exception = False
    try:
        spec = tator.models.Algorithm(
            name=NAME,
            project=project,
            user=user_id,
            description=DESCRIPTION,
            manifest=manifest_url,
            cluster=CLUSTER)

        response = tator_api.register_algorithm(project=project, algorithm_spec=spec)
    except:
        caught_exception = True

    assert caught_exception



import os
import math
from uuid import uuid1

from tusclient.client import TusClient
from urllib.parse import urljoin
from urllib.parse import urlsplit

from .md5sum import md5sum

def upload_temporary_file(api, project, path, lookup=None, hours=24,
                          name=None, chunk_size=100*1024*1024):
    """ Upload a file to the temporary file storage location.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        for progress, response in tator.util.upload_temporary_file(api, project_id, path):
            print(f"Upload progress: {progress}%")
        print(response.message)

    :param api: :class:`tator.TatorApi` object.
    :param project: Unique integer identifying a project.
    :param path: Path to the file.
    :param lookup: [Optional] md5hash of lookup parameters.
    :param hours: [Optional] Number of hourse file is kept alive. Default is 24.
    :param name: [Optional] Name of temporary file in database. Defaults to basename of path.
    :param chunk_size: [Optional] Chunk size in bytes. Default is 100MB.
    :returns: Generator that yields tuple containing progress (0-100) and a
        response. The response is `None` until the last yield, when the response
        is the response object from :meth:`tator.util.TatorApi.create_temporary_file`.
    """
    if name is None:
        name = os.path.basename(path)

    if lookup is None:
        lookup = name

    host = api.api_client.configuration.host
    token = api.api_client.configuration.api_key['Authorization']
    prefix = api.api_client.configuration.api_key_prefix['Authorization']
    tusURL = urljoin(host, "files/")
    tus = TusClient(tusURL, headers={'Authorization': f'{prefix} {token}',
                                     'Upload-Uid': f'{str(uuid1())}'})
    uploader = tus.uploader(path, chunk_size=chunk_size,
                            retries=10, retry_delay=15)
    last_progress = 0
    yield (last_progress, None)
    num_chunks=math.ceil(uploader.get_file_size()/chunk_size)
    for chunk_count in range(num_chunks):
        uploader.upload_chunk()
        this_progress = round((chunk_count / num_chunks) *100,1)
        if this_progress != last_progress:
            yield (this_progress, None)
            last_progress = this_progress

    response = api.create_temporary_file(project, temporary_file_spec={
        "url": uploader.url,
        "name": name,
        "lookup": lookup,
        "hours": 24,
    })
    yield (100, response)




def chunked_create(func, project, **kwargs):
    """ Breaks a create_*_list operation into chunks.

    Example:

    .. code-block:: python

        created_ids = []
        for response in tator.util.chunked_create(api.create_localization_list, 1,
                                                  localization_spec=my_long_list):
            created_ids += response.id

    :param func: Function to call on each chunk.
    :param project: Unique integer identifying a project.
    :returns: Generator that yields a response.
    """
    for key in kwargs:
        spec = kwargs[key]
    for idx in range(0, len(spec), 500):
        response = func(project, **{key: spec[idx:idx+500]})
        yield response



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class StateUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'dict(str, object)',
        'frame': 'int',
        'localization_ids_add': 'list[int]',
        'localization_ids_remove': 'list[int]'
    }

    attribute_map = {
        'attributes': 'attributes',
        'frame': 'frame',
        'localization_ids_add': 'localization_ids_add',
        'localization_ids_remove': 'localization_ids_remove'
    }

    def __init__(self, attributes=None, frame=None, localization_ids_add=None, localization_ids_remove=None, local_vars_configuration=None):  # noqa: E501
        """StateUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._frame = None
        self._localization_ids_add = None
        self._localization_ids_remove = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        if frame is not None:
            self.frame = frame
        if localization_ids_add is not None:
            self.localization_ids_add = localization_ids_add
        if localization_ids_remove is not None:
            self.localization_ids_remove = localization_ids_remove

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this StateUpdate. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this StateUpdate.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def frame(self):
        """
        Frame number this state applies to.

        :return: The frame of this StateUpdate. 
        :rtype: int
        """
        return self._frame

    @frame.setter
    def frame(self, frame):
        """
        Frame number this state applies to.

        :param frame: The frame of this StateUpdate.
        :type: int
        """

        self._frame = frame

    @property
    def localization_ids_add(self):
        """
        List of new localization IDs that this state applies to.

        :return: The localization_ids_add of this StateUpdate. 
        :rtype: list[int]
        """
        return self._localization_ids_add

    @localization_ids_add.setter
    def localization_ids_add(self, localization_ids_add):
        """
        List of new localization IDs that this state applies to.

        :param localization_ids_add: The localization_ids_add of this StateUpdate.
        :type: list[int]
        """

        self._localization_ids_add = localization_ids_add

    @property
    def localization_ids_remove(self):
        """
        List of new localization IDs that this state applies to.

        :return: The localization_ids_remove of this StateUpdate. 
        :rtype: list[int]
        """
        return self._localization_ids_remove

    @localization_ids_remove.setter
    def localization_ids_remove(self, localization_ids_remove):
        """
        List of new localization IDs that this state applies to.

        :param localization_ids_remove: The localization_ids_remove of this StateUpdate.
        :type: list[int]
        """

        self._localization_ids_remove = localization_ids_remove

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StateUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StateUpdate):
            return True

        return self.to_dict() != other.to_dict()
